from multiprocessing import cpu_count
from multiprocessing.dummy import Pool
from os import remove

from shutil import copyfile

from numpy import concatenate

from ....Classes._FEMMHandler import _FEMMHandler


def solve_FEMM_parallel(
    self,
    femm,
    output,
    out_dict,
    FEMM_dict,
    sym,
    Nt,
    angle,
    Is,
    Ir,
    angle_rotor,
):
    """
    Same as solve_FEMM including parallelization on several workers

    Parameters
    ----------
    self: MagFEMM
        A MagFEMM object
    femm: FEMMHandler
        Object to handle FEMM
    output: Output
        An Output object
    out_dict: dict
        Dict containing the following quantities to update for each time step:
            Br : ndarray
                Airgap radial flux density (Nt,Na) [T]
            Bt : ndarray
                Airgap tangential flux density (Nt,Na) [T]
            Tem : ndarray
                Electromagnetic torque over time (Nt,) [Nm]
            Phi_wind_stator : ndarray
                Stator winding flux (qs,Nt) [Wb]
    FEMM_dict : dict
        Dict containing FEMM model parameters
    sym: int
        Spatial symmetry factor
    Nt: int
        Number of time steps for calculation
    angle: ndarray
        Angle vector for calculation
    Is : ndarray
        Stator current matrix (qs,Nt) [A]
    Ir : ndarray
        Stator current matrix (qs,Nt) [A]
    angle_rotor: ndarray
        Rotor angular position vector (Nt,)

    Returns
    -------
    B: ndarray
        3D Magnetic flux density for all time steps and each element (Nt, Nelem, 3) [T]
    H : ndarray
        3D Magnetic field for all time steps and each element (Nt, Nelem, 3) [A/m]
    mu : ndarray
        Magnetic relative permeability for all time steps and each element (Nt, Nelem) []
    mesh: MeshMat
        Object containing magnetic mesh at first time step
    groups: dict
        Dict whose values are group label and values are array of indices of related elements
    """

    # The following function must be in solve_FEMM_parallel to access
    # to its variable without passing them in arguments
    def solve_FEMM_single(femm_handler, filename, start_t, end_t):
        """
        Call FEMM to compute airgap flux density from start_t to end_t timesteps

        This function is called in threads, the shared memory enable to modify global variable
        defined in solve_FEMM_parallel such as Br and Bt

        Parameters
        ----------

        femm_handler : _FEMMHandler
            FEMM handler
        filename : str
            .fem file path
        start_t : int
            first timestep to compute
        end_t: int
            last timestep to compute

        Returns
        -------
        B: ndarray
            3D Magnetic flux density for time steps of the current FEMM instance and each element (Nt0, Nelem, 3) [T]
        H : ndarray
            3D Magnetic field for time steps of the current FEMM instance and each element (Nt0, Nelem, 3) [A/m]
        mu : ndarray
            Magnetic relative permeability for time steps of the current FEMM instance and each element (Nt0, Nelem) []
        mesh: MeshMat
            Object containing magnetic mesh at first time step of the current FEMM instance
        groups: dict
            Dict whose values are group label and values are array of indices of related elements

        """

        B_elem, H_elem, mu_elem, meshFEMM, groups = self.solve_FEMM(
            femm_handler,
            output,
            out_dict,
            FEMM_dict,
            sym=sym,
            Nt=Nt,
            angle=angle,
            Is=Is,
            Ir=Ir,
            angle_rotor=angle_rotor,
            is_close_femm=True,
            filename=filename,
            start_t=start_t,
            end_t=end_t,
        )

        return B_elem, H_elem, mu_elem, meshFEMM, groups

    # Init mesh solution as None since array allocation can only be done once
    # number of elements is known, i.e. after first time step resolution
    B_elem, H_elem, mu_elem, meshFEMM, groups = None, None, None, None, None

    # Loading parameters for readibility
    fem_file = self.get_path_save_fem(output)
    nb_worker = self.nb_worker
    logger = self.get_logger()

    # Check method parameters
    if nb_worker > cpu_count():
        logger.warning(
            f"Parallelization is set on {nb_worker} threads while "
            + f"your computer only has {cpu_count()}."
        )
    if nb_worker > Nt:
        logger.debug(
            f"{nb_worker} workers requested for {Nt} time steps. "
            + f"Using {Nt} workers instead"
        )
        nb_worker = Nt

    # Copy femm file and create lists to split the tasks
    nb_task_worker = []  # nb of task for each worker
    nb_task_to_split = Nt  # number of task remaining to split
    for w in range(nb_worker, 0, -1):
        # Copy the file
        copyfile(fem_file, fem_file[:-4] + "_" + str(w) + ".fem")
        # Compute the number of task for this worker
        nb_task_worker.append(nb_task_to_split // w + ((nb_task_to_split % w) > 0))
        # Remove the number of task of this worker from the number of remaining tasks
        nb_task_to_split -= nb_task_worker[-1]

    # Define the argument lists:
    #   - femm_handler
    #   - femm_file
    #   - start_time : index of the first time step to compute
    #   - end_time : index of the last time step to compute
    femm_handler = [_FEMMHandler() for i in range(nb_worker)]
    femm_files = [
        fem_file[:-4] + "_" + str(i) + ".fem" for i in range(1, nb_worker + 1)
    ]
    start_time_list = [sum(nb_task_worker[:i]) for i in range(nb_worker)]
    end_time_list = [sum(nb_task_worker[: i + 1]) for i in range(nb_worker)]

    # Gathering the different arguments for each instance
    args = [
        [i, j, k, l]
        for i, j, k, l in zip(femm_handler, femm_files, start_time_list, end_time_list)
    ]

    # append femm_handler to handler_list
    output.mag.internal.handler_list.extend(femm_handler)

    # Creating threads pool
    pool = Pool(nb_worker)

    # Computing FEMM in parallel
    results = pool.starmap(solve_FEMM_single, args)

    # Building mesh solution
    if self.is_get_meshsolution:
        B_elem = concatenate([res[0] for res in results], axis=0)
        H_elem = concatenate([res[1] for res in results], axis=0)
        mu_elem = concatenate([res[2] for res in results], axis=0)
        meshFEMM = results[0][3]
        groups = results[0][4]

    # Remove temporary .fem and .ans files
    for w in range(nb_worker, 0, -1):
        try:
            remove(fem_file[:-4] + "_" + str(w) + ".fem")
        except:
            filename = fem_file[:-4] + "_" + str(w) + ".fem"
            self.get_logger().warning("Could not remove file: " + filename)

        try:
            remove(fem_file[:-4] + "_" + str(w) + ".ans")
        except:
            filename = fem_file[:-4] + "_" + str(w) + ".ans"
            self.get_logger().warning("Could not remove file: " + filename)

    return B_elem, H_elem, mu_elem, meshFEMM, groups



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportMatrixXls.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportMatrixXls
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ImportMatrix import ImportMatrix

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportMatrixXls.get_data import get_data
except ImportError as error:
    get_data = error


from ._check import InitUnKnowClassError


class ImportMatrixXls(ImportMatrix):
    """Import the data from an xls file"""

    VERSION = 1

    # cf Methods.Import.ImportMatrixXls.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportMatrixXls method get_data: " + str(get_data)
                )
            )
        )
    else:
        get_data = get_data
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        file_path="",
        sheet="",
        skiprows=0,
        usecols=None,
        axes_colrows=None,
        is_allsheets=False,
        is_transpose=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "file_path" in list(init_dict.keys()):
                file_path = init_dict["file_path"]
            if "sheet" in list(init_dict.keys()):
                sheet = init_dict["sheet"]
            if "skiprows" in list(init_dict.keys()):
                skiprows = init_dict["skiprows"]
            if "usecols" in list(init_dict.keys()):
                usecols = init_dict["usecols"]
            if "axes_colrows" in list(init_dict.keys()):
                axes_colrows = init_dict["axes_colrows"]
            if "is_allsheets" in list(init_dict.keys()):
                is_allsheets = init_dict["is_allsheets"]
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.file_path = file_path
        self.sheet = sheet
        self.skiprows = skiprows
        self.usecols = usecols
        self.axes_colrows = axes_colrows
        self.is_allsheets = is_allsheets
        # Call ImportMatrix init
        super(ImportMatrixXls, self).__init__(is_transpose=is_transpose)
        # The class is frozen (in ImportMatrix init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportMatrixXls_str = ""
        # Get the properties inherited from ImportMatrix
        ImportMatrixXls_str += super(ImportMatrixXls, self).__str__()
        ImportMatrixXls_str += 'file_path = "' + str(self.file_path) + '"' + linesep
        ImportMatrixXls_str += 'sheet = "' + str(self.sheet) + '"' + linesep
        ImportMatrixXls_str += "skiprows = " + str(self.skiprows) + linesep
        ImportMatrixXls_str += 'usecols = "' + str(self.usecols) + '"' + linesep
        ImportMatrixXls_str += "axes_colrows = " + str(self.axes_colrows) + linesep
        ImportMatrixXls_str += "is_allsheets = " + str(self.is_allsheets) + linesep
        return ImportMatrixXls_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ImportMatrix
        if not super(ImportMatrixXls, self).__eq__(other):
            return False
        if other.file_path != self.file_path:
            return False
        if other.sheet != self.sheet:
            return False
        if other.skiprows != self.skiprows:
            return False
        if other.usecols != self.usecols:
            return False
        if other.axes_colrows != self.axes_colrows:
            return False
        if other.is_allsheets != self.is_allsheets:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ImportMatrix
        diff_list.extend(super(ImportMatrixXls, self).compare(other, name=name))
        if other._file_path != self._file_path:
            diff_list.append(name + ".file_path")
        if other._sheet != self._sheet:
            diff_list.append(name + ".sheet")
        if other._skiprows != self._skiprows:
            diff_list.append(name + ".skiprows")
        if other._usecols != self._usecols:
            diff_list.append(name + ".usecols")
        if other._axes_colrows != self._axes_colrows:
            diff_list.append(name + ".axes_colrows")
        if other._is_allsheets != self._is_allsheets:
            diff_list.append(name + ".is_allsheets")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ImportMatrix
        S += super(ImportMatrixXls, self).__sizeof__()
        S += getsizeof(self.file_path)
        S += getsizeof(self.sheet)
        S += getsizeof(self.skiprows)
        S += getsizeof(self.usecols)
        if self.axes_colrows is not None:
            for key, value in self.axes_colrows.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.is_allsheets)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from ImportMatrix
        ImportMatrixXls_dict = super(ImportMatrixXls, self).as_dict(**kwargs)
        ImportMatrixXls_dict["file_path"] = self.file_path
        ImportMatrixXls_dict["sheet"] = self.sheet
        ImportMatrixXls_dict["skiprows"] = self.skiprows
        ImportMatrixXls_dict["usecols"] = self.usecols
        ImportMatrixXls_dict["axes_colrows"] = (
            self.axes_colrows.copy() if self.axes_colrows is not None else None
        )
        ImportMatrixXls_dict["is_allsheets"] = self.is_allsheets
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportMatrixXls_dict["__class__"] = "ImportMatrixXls"
        return ImportMatrixXls_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.file_path = None
        self.sheet = None
        self.skiprows = None
        self.usecols = None
        self.axes_colrows = None
        self.is_allsheets = None
        # Set to None the properties inherited from ImportMatrix
        super(ImportMatrixXls, self)._set_None()

    def _get_file_path(self):
        """getter of file_path"""
        return self._file_path

    def _set_file_path(self, value):
        """setter of file_path"""
        check_var("file_path", value, "str")
        self._file_path = value

    file_path = property(
        fget=_get_file_path,
        fset=_set_file_path,
        doc=u"""Path of the file to load

        :Type: str
        """,
    )

    def _get_sheet(self):
        """getter of sheet"""
        return self._sheet

    def _set_sheet(self, value):
        """setter of sheet"""
        check_var("sheet", value, "str")
        self._sheet = value

    sheet = property(
        fget=_get_sheet,
        fset=_set_sheet,
        doc=u"""Name of the sheet to load

        :Type: str
        """,
    )

    def _get_skiprows(self):
        """getter of skiprows"""
        return self._skiprows

    def _set_skiprows(self, value):
        """setter of skiprows"""
        check_var("skiprows", value, "int", Vmin=0)
        self._skiprows = value

    skiprows = property(
        fget=_get_skiprows,
        fset=_set_skiprows,
        doc=u"""To skip some rows in the file (header)

        :Type: int
        :min: 0
        """,
    )

    def _get_usecols(self):
        """getter of usecols"""
        return self._usecols

    def _set_usecols(self, value):
        """setter of usecols"""
        check_var("usecols", value, "str")
        self._usecols = value

    usecols = property(
        fget=_get_usecols,
        fset=_set_usecols,
        doc=u"""To select the range of column to use

        :Type: str
        """,
    )

    def _get_axes_colrows(self):
        """getter of axes_colrows"""
        return self._axes_colrows

    def _set_axes_colrows(self, value):
        """setter of axes_colrows"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("axes_colrows", value, "dict")
        self._axes_colrows = value

    axes_colrows = property(
        fget=_get_axes_colrows,
        fset=_set_axes_colrows,
        doc=u"""To read axes in first line/column

        :Type: dict
        """,
    )

    def _get_is_allsheets(self):
        """getter of is_allsheets"""
        return self._is_allsheets

    def _set_is_allsheets(self, value):
        """setter of is_allsheets"""
        check_var("is_allsheets", value, "bool")
        self._is_allsheets = value

    is_allsheets = property(
        fget=_get_is_allsheets,
        fset=_set_is_allsheets,
        doc=u"""To read all sheets in a 3D matrix

        :Type: bool
        """,
    )



from ....Classes.Segment import Segment
from ....definitions import PACKAGE_NAME


def split_line(self, Z1, Z2, is_top=True, is_join=False, label_join=""):
    """Cut the Surface according to a line defined by two complex

    Parameters
    ----------
    self : Surface
        An Surface object
    Z1 : complex
        First point of the cutting Line
    Z2 : complex
        Second point of the cutting Line
    is_top : bool
        True to keep the part above the cutting line.
        "Above" is in the coordinate system with Z1 in 0 and Z2 on the X>0 axis
    is_join : bool
        True to join the split_list with Segment on the cutting line
    label_join : str
        Label of the join line

    Returns
    -------
    split_surf : SurfLine
        The selected part of the Surface
    """

    # Dynamic import to avoid import loop
    module = __import__(PACKAGE_NAME + ".Classes.SurfLine", fromlist=["SurfLine"])
    SurfLine = getattr(module, "SurfLine")

    # Split all the lines of the surface
    lines = self.get_lines()
    split_list = list()
    for line in lines:
        split_list.extend(
            line.split_line(
                Z1=Z1, Z2=Z2, is_top=is_top, is_join=is_join, label_join=label_join
            )
        )

    # Make sure that the surface is closed (if needed)
    if is_join:
        final_list = list()
        for ii in range(len(split_list) - 1):
            final_list.append(split_list[ii])
            if abs(split_list[ii].get_end() - split_list[ii + 1].get_begin()) > 1e-6:
                final_list.append(
                    Segment(
                        begin=split_list[ii].get_end(),
                        end=split_list[ii + 1].get_begin(),
                        label=label_join,
                    )
                )
        final_list.append(split_list[-1])
        # Add last line
        if abs(split_list[-1].get_end() - split_list[0].get_begin()) > 1e-6:
            final_list.append(
                Segment(
                    begin=split_list[-1].get_end(),
                    end=split_list[0].get_begin(),
                    label=label_join,
                )
            )
        split_list = final_list

    # Create the resulting surface and update ref point
    surf = SurfLine(label=self.label, line_list=split_list)
    surf.comp_point_ref(is_set=True)

    return surf



# -*- coding: utf-8 -*-


def comp_radius_mid_yoke(self):
    """Compute the Lamination middle of the yoke radius

    Parameters
    ----------
    self : LamSlot
        A LamSlot object

    Returns
    -------
    Ry: float
        middle of the yoke radius [m]

    """

    Hyoke = self.comp_height_yoke()
    if self.is_internal:
        return self.Rint - Hyoke / 2
    else:
        return self.Rext - Hyoke / 2



# -*- coding: utf-8 -*-

from matplotlib.patches import Circle, Patch
from matplotlib.pyplot import axis, legend, subplots
from numpy import exp, pi, sqrt

from ....definitions import config_dict

COND_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][0].copy()
INS_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][1].copy()
COND_INS_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][2].copy()
# Remove alpha from phases
COND_COLOR[3] = 1
INS_COLOR[3] = 1
COND_INS_COLOR[3] = 1


def plot(self, is_show_fig=True):
    """Plot a Conductor in a matplotlib fig

    Parameters
    ----------
    self : CondType12
        A CondType12 object
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None

    Raises
    _______
    NotPlotableError
        You can't plot a coil with Nwppc>4
    """

    patches_list = []

    # Conductor insultation
    patches_list.append(Circle((0, 0), self.Wins_cond / 2, color=COND_INS_COLOR))

    # Computation of the center of the wire
    center_list = []
    if self.Nwppc == 1:
        center_list.append((0, 0))
    elif self.Nwppc == 2:
        center_list.append((0, self.Wwire / 2 + self.Wins_wire))
        center_list.append((0, -self.Wwire / 2 - self.Wins_wire))
    elif self.Nwppc == 3:
        # The 3 centers are on the edges of an Equilateral Triangle
        # (side length : a = Dwire + 2Wins_wire)
        # The Radius of the circumscribed cercle is : a *sqrt(3)/3
        R = (self.Wwire + 2 * self.Wins_wire) * sqrt(3) / 3.0
        center_list.append((0, R))
        # We found the coordinate of the other center by complex rotation
        Z2 = R * 1j * exp(1j * 2 * pi / 3)
        Z3 = R * 1j * exp(-1j * 2 * pi / 3)
        center_list.append((Z2.real, Z2.imag))
        center_list.append((Z3.real, Z3.imag))
    elif self.Nwppc == 4:
        # The 4 centers are on the edges of a square
        # (side length : a =Dwire + 2Wins_wire)
        a = self.Wwire / 2 + self.Wins_wire
        center_list.append((a, a))
        center_list.append((a, -a))
        center_list.append((-a, a))
        center_list.append((-a, -a))
    else:
        raise NotPlotableError("You can't plot a coil with Nwppc>4")

    # Creation of the wires
    for center in center_list:
        # Wire insulation
        patches_list.append(
            Circle(center, self.Wwire / 2 + self.Wins_wire, color=INS_COLOR)
        )
        # Wire conductor
        patches_list.append(Circle(center, self.Wwire / 2, color=COND_COLOR))

    # Display
    fig, ax = subplots()
    for patch in patches_list:
        ax.add_patch(patch)

    # Axis Setup
    axis("equal")

    # The conductor is centered
    ax_lim = self.Wins_cond / 2 + self.Wins_cond / 10
    ax.set_xlim(-ax_lim, ax_lim)
    ax.set_ylim(-ax_lim, ax_lim)

    # Legend
    patch_leg = list()  # Symbol
    label_leg = list()  # Text
    patch_leg.append(Patch(color=COND_INS_COLOR))
    label_leg.append("Coil insulation")
    patch_leg.append(Patch(color=INS_COLOR))
    label_leg.append("Wire insulation")
    patch_leg.append(Patch(color=COND_COLOR))
    label_leg.append("Active wire section")

    legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()


class NotPlotableError(Exception):
    """ """

    pass



import numpy as np
import itertools
from ....Functions.Simulation.VarSimu.run_multisim_step import run_multisim_step
from ....Functions.Simulation.VarSimu.log_datakeeper_step_result import (
    log_datakeeper_step_result,
)
from ....Functions.Load.import_class import import_class


def run(self):
    """Run the multi-simulation
    The VarSimu object must be in a Simulation object (and not a VarSimu one)

    Parameters
    ----------
    self : VarSimu
        A VarSimu object
    """

    logger = self.get_logger()
    Output = import_class("pyleecan.Classes", "Output")

    # Check var_simu parameters
    self.check_param()  # Check isinstance(self.parent, Simulation)

    # Get xoutput to store results (created in Simulation.run)
    xoutput = self.parent.parent

    # Create reference simulation
    ref_simu = self.parent.copy(keep_function=True)
    ref_simu.var_simu = self.var_simu  # var_simu default is None
    ref_simu.index = None
    ref_simu.layer = self.parent.layer + 1

    # Generate simulation list and ParamExplorerValue list
    simu_dict = self.generate_simulation_list(ref_simu)

    # Check if one of the simulation from the list matches the reference one
    ref_simu_index = None
    for ii, simu in enumerate(simu_dict["simulation_list"]):
        if simu == ref_simu:
            ref_simu_index = ii
            break
    if self.NAME == "Variable Load" and ref_simu_index is None:
        logger.warning(
            "Reference Operating point is not in OP_matrix, one extra simulation will be computed"
        )

    # Fill/initialize input/output parameters
    nb_simu = len(simu_dict["simulation_list"])
    xoutput.nb_simu = nb_simu
    self.nb_simu = nb_simu
    xoutput.paramexplorer_list = simu_dict["paramexplorer_list"]
    simulation_list = simu_dict["simulation_list"]
    if self.is_keep_all_output:
        xoutput.output_list = [None] * self.nb_simu
    keeper_list = list()  # Copy keeper to store results in xoutput and not in simu
    for datakeeper in self.datakeeper_list:
        keeper_list.append(datakeeper.copy(keep_function=True))
        xoutput.xoutput_dict[datakeeper.symbol] = keeper_list[-1]
        keeper_list[-1].result = [None] * self.nb_simu

    # Run Reference simulation
    if ref_simu.layer == 2:
        logger.info("    Computing reference simulation for " + self.NAME)
    else:
        logger.info("Computing reference simulation for " + self.NAME)

    progress = 0  # For progress bar
    nb_simu += 1  # Count reference simulation in progress bar

    # Run the simulation & call DataKeeper and post-proc handling errors
    xoutput_ref = run_multisim_step(
        ref_simu,
        datakeeper_list=keeper_list,  # datakeeper.result will be updated (if needed)
        stop_if_error=True,  # Reference simulation must be correct
        post_keeper_postproc_list=self.post_keeper_postproc_list,
        simu_type=self.NAME,
    )
    # Save reference xoutput (if requested)
    if self.is_keep_all_output:
        if ref_simu_index is None:
            xoutput.xoutput_ref = xoutput_ref
        else:  # Avoid duplicating the XOutput
            xoutput.xoutput_ref_index = ref_simu_index

    progress += 1
    print_progress_bar(nb_simu, progress, ref_simu.layer)

    # Store the Ouput part of the xoutput_ref into the main xoutput
    ref_out_dict = Output.as_dict(xoutput_ref)
    if "simu" in ref_out_dict:  # Avoid erasing original simulation
        ref_out_dict.pop("simu")
    ref_out_dict.pop("__class__")
    for key, value in ref_out_dict.items():
        setattr(xoutput, key, value)

    # Reuse some intermediate results from reference simulation (if requested)
    for ii, simu in enumerate(simulation_list):
        # Log only for first simulation
        self.set_reused_data(
            simu,
            xoutput_ref,
            is_log=ii == 0,
            simu_type=self.NAME,
        )

    # Update the postprocessing list if needed
    if self.pre_keeper_postproc_list is not None:
        # Different post between simu list and ref simu
        for simu in simulation_list:
            simu.postproc_list = self.pre_keeper_postproc_list

    # Execute the simulation list
    for idx, simu_step in enumerate(simulation_list):
        # Display simulation progress
        log_step_simu(
            idx, self.nb_simu, xoutput.paramexplorer_list, logger, simu_step.layer
        )
        simu_step.index = idx
        if idx != ref_simu_index:
            # Run the simulation & call DataKeeper and post-proc handling errors
            xoutput_step = run_multisim_step(
                simu_step,
                keeper_list,  # datakeeper.result will be updated (if needed)
                self.stop_if_error,
                post_keeper_postproc_list=self.post_keeper_postproc_list,
                simu_type=self.NAME,
            )
            if self.is_keep_all_output:
                xoutput.output_list[idx] = xoutput_step
        else:
            if simu_step.layer == 2:
                logger.info(
                    "    Simulation matches reference one: Skipping computation"
                )
            else:
                logger.info("Simulation matches reference one: Skipping computation")
            # Copy results from reference
            for keeper in keeper_list:
                keeper.result[idx] = keeper.result_ref
            if self.is_keep_all_output:
                xoutput.output_list[idx] = xoutput_ref
            # Print DataKeeper content
            log_datakeeper_step_result(simu_step, keeper_list, idx, self.NAME)
        progress += 1
        print_progress_bar(nb_simu, progress, simu_step.layer)

    # Running postprocessings
    if self.postproc_list:
        logger.info("Running " + self.NAME + " postprocessings...")
        for postproc in self.postproc_list:
            postproc.run(xoutput)


def log_step_simu(index, nb_simu, paramexplorer_list, logger, layer):
    """Add in the log some information about the simulation about to run
    Ex: "Running simulation 3/4 with Id=-135.41881, Iq=113.62987"

    Parameters
    ----------
    index : int
        Index of the simulation about to run
    nb_simu : int
        Total number of simulation to run
    paramexplorer_list : list
        List of ParamExplorer to get the modification of the simu
    logger : Logger
        Logger to use (info)
    """

    if layer == 2:
        msg = "    "
    else:
        msg = ""
    InputCurrent = import_class("pyleecan.Classes", "InputCurrent")
    msg += "Running simulation " + str(index + 1) + "/" + str(nb_simu) + " with "
    for param_exp in paramexplorer_list:
        value = param_exp.get_value()[index]
        if isinstance(value, InputCurrent):
            msg += "N0=" + format(value.N0, ".6g") + " [rpm]"
            msg += ", Id=" + format(value.Id_ref, ".4g") + " [Arms]"
            msg += ", Iq=" + format(value.Iq_ref, ".4g") + " [Arms], "
        elif isinstance(value, (list, np.ndarray)):
            msg += param_exp.symbol
            msg += "="
            msg += format(value)
            msg += ", "
        else:
            try:
                data_str = format(value, ".8g")
            except:
                data_str = "'NA'"
            msg += param_exp.symbol
            msg += "="
            msg += data_str
            msg += ", "
    msg = msg[:-2]
    logger.info(msg)


def print_progress_bar(Nsimu, index, layer):
    """Print the progress bar
    Ex: "[=========================                         ]  50%"

    Parameters
    ----------
    Nsimu : int
        Total number of simulation
    index : int
        Index of the simulation about to run
    """
    if layer == 2:
        msg = "               "
    else:
        msg = ""
    print(
        "\r"
        + msg
        + "["
        + "=" * (50 * index // (Nsimu))
        + " " * (50 - ((50 * index) // (Nsimu)))
        + "] {:3d}%".format(((100 * index) // (Nsimu)))
    )



# -*- coding: utf-8 -*-
"""File generated according to PVentPolar/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentPolar.Ui_PVentPolar import (
    Ui_PVentPolar,
)


class Gen_PVentPolar(Ui_PVentPolar):
    def setupUi(self, PVentPolar):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PVentPolar.setupUi(self, PVentPolar)
        # Setup of in_Alpha0
        txt = self.tr(u"""Shift angle of the hole around circumference""")
        self.in_Alpha0.setWhatsThis(txt)
        self.in_Alpha0.setToolTip(txt)

        # Setup of lf_Alpha0
        self.lf_Alpha0.validator().setBottom(0)
        self.lf_Alpha0.validator().setTop(6.29)
        txt = self.tr(u"""Shift angle of the hole around circumference""")
        self.lf_Alpha0.setWhatsThis(txt)
        self.lf_Alpha0.setToolTip(txt)

        # Setup of in_D0
        txt = self.tr(u"""Height of the hole""")
        self.in_D0.setWhatsThis(txt)
        self.in_D0.setToolTip(txt)

        # Setup of lf_D0
        self.lf_D0.validator().setBottom(0)
        txt = self.tr(u"""Height of the hole""")
        self.lf_D0.setWhatsThis(txt)
        self.lf_D0.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Radius of the bottom of Hole""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Radius of the bottom of Hole""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_W1
        txt = self.tr(u"""Hole angular width""")
        self.in_W1.setWhatsThis(txt)
        self.in_W1.setToolTip(txt)

        # Setup of lf_W1
        self.lf_W1.validator().setBottom(0)
        self.lf_W1.validator().setTop(6.29)
        txt = self.tr(u"""Hole angular width""")
        self.lf_W1.setWhatsThis(txt)
        self.lf_W1.setToolTip(txt)



# -*- coding: utf-8 -*-

from ....Methods.Machine.Lamination import *


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self :
        A Lamination object

    Returns
    -------
    None

    Raises
    _______
    LC_VentYokeIn
        The ventilation ducts cross the yoke internal radius
    LC_VentYokeOut
        The ventilation ducts cross the yoke external radius
    LC_RadiusError
        The internal radius is greater than the external one


    """

    if self.Rint > self.Rext:
        raise LC_RadiusError("The internal radius is greater than the " "external one")

    for vent in self.axial_vent:
        vent.check()

        # The ventilation duct must be on the yoke
        if vent.Zh > 0:
            if self.is_internal:
                Rin = self.Rint
                Rout = self.Rint + self.comp_height_yoke()
            else:
                Rin = self.Rint + self.comp_height_yoke()
                Rout = self.Rext

            (Rmin, Rmax) = vent.comp_radius()

            if Rmin < Rin:
                raise LC_VentYokeIn(
                    "The ventilation ducts cross the yoke " "internal radius"
                )
            if Rmax > Rout:
                raise LC_VentYokeOut(
                    "The ventilation ducts cross the yoke " "external radius"
                )



class PointSegmentError(Exception):
    """ """

    pass


class AngleRotationSegmentError(Exception):
    """ """

    pass


class PointTranslateSegmentError(Exception):
    """ """

    pass


class NbPointSegmentDError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/LossModelBertotti.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/LossModelBertotti
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .LossModel import LossModel

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.LossModelBertotti.comp_loss import comp_loss
except ImportError as error:
    comp_loss = error

try:
    from ..Methods.Simulation.LossModelBertotti.comp_coeff_Bertotti import (
        comp_coeff_Bertotti,
    )
except ImportError as error:
    comp_coeff_Bertotti = error

try:
    from ..Methods.Simulation.LossModelBertotti.comp_loss_density import (
        comp_loss_density,
    )
except ImportError as error:
    comp_loss_density = error


from ._check import InitUnKnowClassError


class LossModelBertotti(LossModel):
    """Bertotti Loss Model Class"""

    VERSION = 1
    F_REF = 50
    B_REF = 1.5

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.LossModelBertotti.comp_loss
    if isinstance(comp_loss, ImportError):
        comp_loss = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LossModelBertotti method comp_loss: " + str(comp_loss)
                )
            )
        )
    else:
        comp_loss = comp_loss
    # cf Methods.Simulation.LossModelBertotti.comp_coeff_Bertotti
    if isinstance(comp_coeff_Bertotti, ImportError):
        comp_coeff_Bertotti = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LossModelBertotti method comp_coeff_Bertotti: "
                    + str(comp_coeff_Bertotti)
                )
            )
        )
    else:
        comp_coeff_Bertotti = comp_coeff_Bertotti
    # cf Methods.Simulation.LossModelBertotti.comp_loss_density
    if isinstance(comp_loss_density, ImportError):
        comp_loss_density = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LossModelBertotti method comp_loss_density: "
                    + str(comp_loss_density)
                )
            )
        )
    else:
        comp_loss_density = comp_loss_density
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        k_hy=None,
        k_ed=None,
        k_ex=None,
        alpha_hy=None,
        alpha_ed=None,
        alpha_ex=None,
        group="core",
        get_meshsolution=False,
        N0=-1,
        name="",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "k_hy" in list(init_dict.keys()):
                k_hy = init_dict["k_hy"]
            if "k_ed" in list(init_dict.keys()):
                k_ed = init_dict["k_ed"]
            if "k_ex" in list(init_dict.keys()):
                k_ex = init_dict["k_ex"]
            if "alpha_hy" in list(init_dict.keys()):
                alpha_hy = init_dict["alpha_hy"]
            if "alpha_ed" in list(init_dict.keys()):
                alpha_ed = init_dict["alpha_ed"]
            if "alpha_ex" in list(init_dict.keys()):
                alpha_ex = init_dict["alpha_ex"]
            if "group" in list(init_dict.keys()):
                group = init_dict["group"]
            if "get_meshsolution" in list(init_dict.keys()):
                get_meshsolution = init_dict["get_meshsolution"]
            if "N0" in list(init_dict.keys()):
                N0 = init_dict["N0"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
        # Set the properties (value check and convertion are done in setter)
        self.k_hy = k_hy
        self.k_ed = k_ed
        self.k_ex = k_ex
        self.alpha_hy = alpha_hy
        self.alpha_ed = alpha_ed
        self.alpha_ex = alpha_ex
        self.group = group
        self.get_meshsolution = get_meshsolution
        self.N0 = N0
        # Call LossModel init
        super(LossModelBertotti, self).__init__(name=name)
        # The class is frozen (in LossModel init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        LossModelBertotti_str = ""
        # Get the properties inherited from LossModel
        LossModelBertotti_str += super(LossModelBertotti, self).__str__()
        LossModelBertotti_str += "k_hy = " + str(self.k_hy) + linesep
        LossModelBertotti_str += "k_ed = " + str(self.k_ed) + linesep
        LossModelBertotti_str += "k_ex = " + str(self.k_ex) + linesep
        LossModelBertotti_str += "alpha_hy = " + str(self.alpha_hy) + linesep
        LossModelBertotti_str += "alpha_ed = " + str(self.alpha_ed) + linesep
        LossModelBertotti_str += "alpha_ex = " + str(self.alpha_ex) + linesep
        LossModelBertotti_str += 'group = "' + str(self.group) + '"' + linesep
        LossModelBertotti_str += (
            "get_meshsolution = " + str(self.get_meshsolution) + linesep
        )
        LossModelBertotti_str += (
            "N0 = " + linesep + str(self.N0).replace(linesep, linesep + "\t") + linesep
        )
        return LossModelBertotti_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from LossModel
        if not super(LossModelBertotti, self).__eq__(other):
            return False
        if other.k_hy != self.k_hy:
            return False
        if other.k_ed != self.k_ed:
            return False
        if other.k_ex != self.k_ex:
            return False
        if other.alpha_hy != self.alpha_hy:
            return False
        if other.alpha_ed != self.alpha_ed:
            return False
        if other.alpha_ex != self.alpha_ex:
            return False
        if other.group != self.group:
            return False
        if other.get_meshsolution != self.get_meshsolution:
            return False
        if other.N0 != self.N0:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from LossModel
        diff_list.extend(super(LossModelBertotti, self).compare(other, name=name))
        if other._k_hy != self._k_hy:
            diff_list.append(name + ".k_hy")
        if other._k_ed != self._k_ed:
            diff_list.append(name + ".k_ed")
        if other._k_ex != self._k_ex:
            diff_list.append(name + ".k_ex")
        if other._alpha_hy != self._alpha_hy:
            diff_list.append(name + ".alpha_hy")
        if other._alpha_ed != self._alpha_ed:
            diff_list.append(name + ".alpha_ed")
        if other._alpha_ex != self._alpha_ex:
            diff_list.append(name + ".alpha_ex")
        if other._group != self._group:
            diff_list.append(name + ".group")
        if other._get_meshsolution != self._get_meshsolution:
            diff_list.append(name + ".get_meshsolution")
        if other._N0 != self._N0:
            diff_list.append(name + ".N0")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from LossModel
        S += super(LossModelBertotti, self).__sizeof__()
        S += getsizeof(self.k_hy)
        S += getsizeof(self.k_ed)
        S += getsizeof(self.k_ex)
        S += getsizeof(self.alpha_hy)
        S += getsizeof(self.alpha_ed)
        S += getsizeof(self.alpha_ex)
        S += getsizeof(self.group)
        S += getsizeof(self.get_meshsolution)
        if self.N0 is not None:
            for value in self.N0:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from LossModel
        LossModelBertotti_dict = super(LossModelBertotti, self).as_dict(**kwargs)
        LossModelBertotti_dict["k_hy"] = self.k_hy
        LossModelBertotti_dict["k_ed"] = self.k_ed
        LossModelBertotti_dict["k_ex"] = self.k_ex
        LossModelBertotti_dict["alpha_hy"] = self.alpha_hy
        LossModelBertotti_dict["alpha_ed"] = self.alpha_ed
        LossModelBertotti_dict["alpha_ex"] = self.alpha_ex
        LossModelBertotti_dict["group"] = self.group
        LossModelBertotti_dict["get_meshsolution"] = self.get_meshsolution
        LossModelBertotti_dict["N0"] = self.N0.copy() if self.N0 is not None else None
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        LossModelBertotti_dict["__class__"] = "LossModelBertotti"
        return LossModelBertotti_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.k_hy = None
        self.k_ed = None
        self.k_ex = None
        self.alpha_hy = None
        self.alpha_ed = None
        self.alpha_ex = None
        self.group = None
        self.get_meshsolution = None
        self.N0 = None
        # Set to None the properties inherited from LossModel
        super(LossModelBertotti, self)._set_None()

    def _get_k_hy(self):
        """getter of k_hy"""
        return self._k_hy

    def _set_k_hy(self, value):
        """setter of k_hy"""
        check_var("k_hy", value, "float")
        self._k_hy = value

    k_hy = property(
        fget=_get_k_hy,
        fset=_set_k_hy,
        doc=u"""Hysteresis loss coefficient

        :Type: float
        """,
    )

    def _get_k_ed(self):
        """getter of k_ed"""
        return self._k_ed

    def _set_k_ed(self, value):
        """setter of k_ed"""
        check_var("k_ed", value, "float")
        self._k_ed = value

    k_ed = property(
        fget=_get_k_ed,
        fset=_set_k_ed,
        doc=u"""Eddy current loss coefficient

        :Type: float
        """,
    )

    def _get_k_ex(self):
        """getter of k_ex"""
        return self._k_ex

    def _set_k_ex(self, value):
        """setter of k_ex"""
        check_var("k_ex", value, "float")
        self._k_ex = value

    k_ex = property(
        fget=_get_k_ex,
        fset=_set_k_ex,
        doc=u"""Excess loss coefficient

        :Type: float
        """,
    )

    def _get_alpha_hy(self):
        """getter of alpha_hy"""
        return self._alpha_hy

    def _set_alpha_hy(self, value):
        """setter of alpha_hy"""
        check_var("alpha_hy", value, "float")
        self._alpha_hy = value

    alpha_hy = property(
        fget=_get_alpha_hy,
        fset=_set_alpha_hy,
        doc=u"""Hysteresis loss power coefficient

        :Type: float
        """,
    )

    def _get_alpha_ed(self):
        """getter of alpha_ed"""
        return self._alpha_ed

    def _set_alpha_ed(self, value):
        """setter of alpha_ed"""
        check_var("alpha_ed", value, "float")
        self._alpha_ed = value

    alpha_ed = property(
        fget=_get_alpha_ed,
        fset=_set_alpha_ed,
        doc=u"""Eddy current loss power coefficient

        :Type: float
        """,
    )

    def _get_alpha_ex(self):
        """getter of alpha_ex"""
        return self._alpha_ex

    def _set_alpha_ex(self, value):
        """setter of alpha_ex"""
        check_var("alpha_ex", value, "float")
        self._alpha_ex = value

    alpha_ex = property(
        fget=_get_alpha_ex,
        fset=_set_alpha_ex,
        doc=u"""Excess loss power coefficient

        :Type: float
        """,
    )

    def _get_group(self):
        """getter of group"""
        return self._group

    def _set_group(self, value):
        """setter of group"""
        check_var("group", value, "str")
        self._group = value

    group = property(
        fget=_get_group,
        fset=_set_group,
        doc=u"""String to override default FEA group to apply model

        :Type: str
        """,
    )

    def _get_get_meshsolution(self):
        """getter of get_meshsolution"""
        return self._get_meshsolution

    def _set_get_meshsolution(self, value):
        """setter of get_meshsolution"""
        check_var("get_meshsolution", value, "bool")
        self._get_meshsolution = value

    get_meshsolution = property(
        fget=_get_get_meshsolution,
        fset=_set_get_meshsolution,
        doc=u"""Store the loss density

        :Type: bool
        """,
    )

    def _get_N0(self):
        """getter of N0"""
        return self._N0

    def _set_N0(self, value):
        """setter of N0"""
        if type(value) is int and value == -1:
            value = list()
        check_var("N0", value, "list")
        self._N0 = value

    N0 = property(
        fget=_get_N0,
        fset=_set_N0,
        doc=u"""List of rotor speeds to override actual speed

        :Type: list
        """,
    )



# -*- coding: utf-8 -*-

from numpy import roll


def shift_wind_mat(wind_mat, Nshift):
    """Shift the Winding Matrix of Nshift number of Slot

    Parameters
    ----------
    wind_mat : numpy.ndarray
        A Winding Matrix (Nlay_r, Nlay_theta, Zs,
        qs) (created by comp_connection_mat)
    Nshift : int
        Number of Slot to shift (Integer 0 < Nshift < Zs)

    Returns
    -------
    shift_wind_mat: numpy.ndarray
        The shifted matrix along the Slot

    """
    assert len(wind_mat.shape) == 4, "wind_mat has a wrong shape (dim 4)"
    assert Nshift % 1 == 0, "Nshift must be an integer"

    return roll(wind_mat, shift=Nshift, axis=2)



# -*- coding: utf-8 -*-

# File generated according to PMSlot17.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit
from ......GUI.Dialog.DMachineSetup.SWSlot.WWSlotOut.WWSlotOut import WWSlotOut

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PMSlot17(object):
    def setupUi(self, PMSlot17):
        if not PMSlot17.objectName():
            PMSlot17.setObjectName(u"PMSlot17")
        PMSlot17.resize(1055, 470)
        PMSlot17.setMinimumSize(QSize(630, 470))
        PMSlot17.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PMSlot17)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.img_slot = QLabel(PMSlot17)
        self.img_slot.setObjectName(u"img_slot")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.img_slot.sizePolicy().hasHeightForWidth())
        self.img_slot.setSizePolicy(sizePolicy)
        self.img_slot.setMaximumSize(QSize(16777215, 16777215))
        self.img_slot.setPixmap(
            QPixmap(u":/images/images/MachineSetup/WMSlot/SlotM17.png")
        )
        self.img_slot.setScaledContents(True)

        self.verticalLayout_2.addWidget(self.img_slot)

        self.txt_constraint = QTextEdit(PMSlot17)
        self.txt_constraint.setObjectName(u"txt_constraint")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(
            self.txt_constraint.sizePolicy().hasHeightForWidth()
        )
        self.txt_constraint.setSizePolicy(sizePolicy1)
        self.txt_constraint.setMaximumSize(QSize(16777215, 70))
        self.txt_constraint.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.txt_constraint.setTextInteractionFlags(
            Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse
        )

        self.verticalLayout_2.addWidget(self.txt_constraint)

        self.horizontalLayout.addLayout(self.verticalLayout_2)

        self.widget = QWidget(PMSlot17)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout = QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.unit_Lmag = QLabel(self.widget)
        self.unit_Lmag.setObjectName(u"unit_Lmag")

        self.gridLayout.addWidget(self.unit_Lmag, 0, 2, 1, 1)

        self.lf_Lmag = FloatEdit(self.widget)
        self.lf_Lmag.setObjectName(u"lf_Lmag")

        self.gridLayout.addWidget(self.lf_Lmag, 0, 1, 1, 1)

        self.in_Lmag = QLabel(self.widget)
        self.in_Lmag.setObjectName(u"in_Lmag")

        self.gridLayout.addWidget(self.in_Lmag, 0, 0, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.w_out = WWSlotOut(self.widget)
        self.w_out.setObjectName(u"w_out")

        self.verticalLayout.addWidget(self.w_out)

        self.horizontalLayout.addWidget(self.widget)

        QWidget.setTabOrder(self.lf_Lmag, self.txt_constraint)

        self.retranslateUi(PMSlot17)

        QMetaObject.connectSlotsByName(PMSlot17)

    # setupUi

    def retranslateUi(self, PMSlot17):
        PMSlot17.setWindowTitle(QCoreApplication.translate("PMSlot17", u"Form", None))
        self.img_slot.setText("")
        self.txt_constraint.setHtml(
            QCoreApplication.translate(
                "PMSlot17",
                u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:7.8pt; font-weight:400; font-style:normal;\">\n"
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:12pt; font-weight:600; text-decoration: underline;">Constraints :</span></p>\n'
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:14pt;">p=1</span></p></body></html>',
                None,
            )
        )
        self.unit_Lmag.setText(QCoreApplication.translate("PMSlot17", u"[m]", None))
        self.in_Lmag.setText(QCoreApplication.translate("PMSlot17", u"Lmag", None))

    # retranslateUi



# -*- coding: utf-8 -*-


def comp_surface(self, Ndisc=200):
    """Compute the Slot total surface (by numerical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : Slot
        A Slot object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    S: float
        Slot total surface [m**2]
    """

    surf = self.get_surface()
    return surf.comp_surface(Ndisc=Ndisc)



# -*- coding: utf-8 -*-

# File generated according to WImport.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_WImport(object):
    def setupUi(self, WImport):
        if not WImport.objectName():
            WImport.setObjectName(u"WImport")
        WImport.resize(678, 491)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WImport.sizePolicy().hasHeightForWidth())
        WImport.setSizePolicy(sizePolicy)
        WImport.setMinimumSize(QSize(0, 0))
        self.main_layout = QVBoxLayout(WImport)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_param = QLabel(WImport)
        self.in_param.setObjectName(u"in_param")

        self.horizontalLayout.addWidget(self.in_param)

        self.c_type_import = QComboBox(WImport)
        self.c_type_import.setObjectName(u"c_type_import")

        self.horizontalLayout.addWidget(self.c_type_import)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_import = QWidget(WImport)
        self.w_import.setObjectName(u"w_import")

        self.main_layout.addWidget(self.w_import)

        self.retranslateUi(WImport)

        QMetaObject.connectSlotsByName(WImport)

    # setupUi

    def retranslateUi(self, WImport):
        WImport.setWindowTitle(QCoreApplication.translate("WImport", u"Form", None))
        self.in_param.setText(
            QCoreApplication.translate("WImport", u"Param_name: ", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

from numpy import abs as np_abs


def comp_height_active(self):
    """Compute the height of the active area

    Parameters
    ----------
    self : SlotM11
        A SlotM11 object

    Returns
    -------
    Hwind: float
        Height of the active area [m]

    """

    return self.Hmag



# -*- coding: utf-8 -*-


from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget, QSizePolicy

from .....Classes.HoleM50 import HoleM50
from .....Classes.Material import Material
from .....GUI.Dialog.DMachineSetup.SMHoleMag.Ui_SMHoleMag import Ui_SMHoleMag
from .....GUI.Dialog.DMachineSetup.SMHoleMag.WHoleMag.WHoleMag import WHoleMag
from .....Methods.Slot.Slot import SlotCheckError


class SMHoleMag(Ui_SMHoleMag, QWidget):
    """Step to set several Holes"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup
    step_name = "Slot"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SMHoleMag
            A SMHoleMag widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Get the correct object to set
        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor
        if self.obj.hole is None:
            self.obj.hole = list()

        # If the hole is not set, initialize it with a HoleM50
        if len(self.obj.hole) == 0:
            self.obj.hole.append(HoleM50())
            if self.machine.type_machine == 5:  # SyRM
                self.obj.hole[0].remove_magnet()
            self.obj.hole[0]._set_None()
            self.obj.hole[0].Zh = machine.stator.winding.p * 2  # Default value
        self.set_hole_pitch(self.obj.hole[0].Zh)

        # Update all the hole tab
        # (the current hole types will be initialized)
        # print(type(self.obj.hole[0]).__name__)
        self.tab_hole.clear()
        for hole in self.obj.hole:
            self.s_add(hole)
        self.tab_hole.setCurrentIndex(0)

        # Set Help URL
        self.b_help.hide()

        # Connect the slot
        self.b_add.clicked.connect(self.s_add)
        self.b_remove.clicked.connect(self.s_remove)

        self.b_plot.clicked.connect(self.s_plot)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()

    def set_hole_pitch(self, Zh):
        """Update out_hole_pitch with the correct value

        Parameters
        ----------
        self : SMHoleMag
            A SMHoleMag object
        Zh : int
            The current value of Zh
        """
        Zh_txt = self.tr("Slot pitch = 360 / 2p = ")
        if Zh in [None, 0]:
            self.out_hole_pitch.setText(Zh_txt + "?")
        else:
            hole_pitch = 360.0 / Zh
            self.out_hole_pitch.setText(Zh_txt + "%.4g" % (hole_pitch) + u" °")

    def s_add(self, hole=False):
        """Signal to add a new hole

        Parameters
        ----------
        self : SMHoleMag
            a SMHoleMag object
        hole : HoleMag
            hole to initialize in the new page
            if None create a new empty HoleM50
        """
        # Adapt the GUI according to the machine type
        if self.machine.type_machine == 5:  # SyRM
            is_mag = False
        else:
            is_mag = True
        # Create a new hole if needed
        if type(hole) is bool:
            self.obj.hole.append(HoleM50())
            hole = self.obj.hole[-1]
            hole._set_None()
            hole_index = len(self.obj.hole) - 1
            if self.machine.type_machine == 5:
                hole.remove_magnet()
            else:
                hole.set_magnet_by_id(0, self.obj.hole[0].get_magnet_by_id(0))
            hole.Zh = self.obj.hole[0].Zh
        else:
            hole_index = self.obj.hole.index(hole)
        tab = WHoleMag(self, is_mag=is_mag, index=hole_index, matlib=self.matlib)
        tab.saveNeeded.connect(self.emit_save)
        self.tab_hole.addTab(tab, "Slot " + str(hole_index + 1))

    def s_remove(self):
        """Signal to remove the last hole

        Parameters
        ----------
        self : SMHoleMag
            a SMHoleMag object
        """
        if len(self.obj.hole) > 1:
            self.tab_hole.removeTab(len(self.obj.hole) - 1)
            self.obj.hole.pop(-1)

    def s_plot(self):
        """Try to plot the lamination

        Parameters
        ----------
        self : SMHoleMag
            a SMHoleMag object
        """
        # Update p
        for hole in self.obj.hole:
            hole.Zh = self.machine.stator.winding.p * 2
        self.set_hole_pitch(self.obj.hole[0].Zh)

        # We have to make sure the hole is right before truing to plot it
        error = self.check(self.obj)
        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the hole (No winding for LamSquirrelCage)
            self.machine.plot(is_show_fig=True)

    @staticmethod
    def check(lamination):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        # Check that everything is set
        for hole in lamination.hole:
            try:
                hole.check()
            except SlotCheckError as error:
                return str(error)



# -*- coding: utf-8 -*-

from matplotlib.pyplot import subplots
from ....Functions.init_fig import init_fig


def plot_BH(self, fig=None, grid=True):
    """Plot the curve B(H) at the specified frequency

    Parameters
    ----------
    self : MatMagnetics
        a MatMagnetics object
    fig :
        if None, open a new fig and plot, else add to the gcf (Default value = None)

    Returns
    -------
    None
    """
    (fig, axes, patch_leg, label_leg) = init_fig(fig)

    if self.BH_curve is not None:
        BH = self.get_BH()
        axes.plot(BH[:, 0], BH[:, 1], color="r")
        axes.grid(b=True)
        axes.set_xlabel("H [A/m]")
        axes.set_ylabel("B [T]")
        axes.set_title("B(H) curve")
        fig.show()



# -*- coding: utf-8 -*-
from numpy import pi

from ....Classes.LamSlot import LamSlot


def build_geometry(self, is_magnet=True, sym=1, alpha=0, delta=0, is_simplified=False):
    """Build the geometry of the LamSlotMag

    Parameters
    ----------
    self : LamSlotMag
        LamSlotMag object
    is_magnet : bool
        If True build the magnet surfaces
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    if self.is_stator:
        st = "Stator"
    else:
        st = "Rotor"

    assert (self.slot.Zs % sym) == 0, (
        "ERROR, Wrong symmetry for "
        + st
        + " "
        + str(self.slot.Zs)
        + " slots and sym="
        + str(sym)
    )
    # getting the LamSlot surface
    surf_list = LamSlot.build_geometry(self, sym=sym)

    Zs = self.slot.Zs
    slot_pitch = 2 * pi / Zs

    # Add the magnet surface(s)
    if is_magnet and self.magnet is not None:
        # for each magnet to draw
        for ii in range(Zs // sym):
            mag_surf = self.slot.get_surface_active(
                alpha=slot_pitch * ii + slot_pitch * 0.5
            )
            # Defining type of magnetization of the magnet
            if self.magnet.type_magnetization == 0:
                type_mag = "Radial"
            elif self.magnet.type_magnetization == 1:
                type_mag = "Parallel"
            elif self.magnet.type_magnetization == 2:
                type_mag = "Hallbach"
            else:
                type_mag = ""

            surf_list.append(mag_surf)
            # Adapt the label
            if ii % 2 != 0:  # South pole
                surf_list[-1].label = (
                    "Magnet" + st + type_mag + "_S_R0" + "_T0_S" + str(ii)
                )
            else:  # North pole
                surf_list[-1].label = (
                    "Magnet" + st + type_mag + "_N_R0" + "_T0_S" + str(ii)
                )

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-

# File generated according to WImportLinspace.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Tools.FloatEdit import FloatEdit

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_WImportLinspace(object):
    def setupUi(self, WImportLinspace):
        if not WImportLinspace.objectName():
            WImportLinspace.setObjectName(u"WImportLinspace")
        WImportLinspace.resize(330, 73)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WImportLinspace.sizePolicy().hasHeightForWidth())
        WImportLinspace.setSizePolicy(sizePolicy)
        WImportLinspace.setMinimumSize(QSize(0, 0))
        self.verticalLayout = QVBoxLayout(WImportLinspace)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.c_type_lin = QComboBox(WImportLinspace)
        self.c_type_lin.addItem("")
        self.c_type_lin.addItem("")
        self.c_type_lin.setObjectName(u"c_type_lin")

        self.horizontalLayout.addWidget(self.c_type_lin)

        self.is_end = QCheckBox(WImportLinspace)
        self.is_end.setObjectName(u"is_end")

        self.horizontalLayout.addWidget(self.is_end)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.in_start = QLabel(WImportLinspace)
        self.in_start.setObjectName(u"in_start")

        self.horizontalLayout_2.addWidget(self.in_start)

        self.lf_start = FloatEdit(WImportLinspace)
        self.lf_start.setObjectName(u"lf_start")
        self.lf_start.setMinimumSize(QSize(40, 0))

        self.horizontalLayout_2.addWidget(self.lf_start)

        self.in_N = QLabel(WImportLinspace)
        self.in_N.setObjectName(u"in_N")

        self.horizontalLayout_2.addWidget(self.in_N)

        self.si_N = QSpinBox(WImportLinspace)
        self.si_N.setObjectName(u"si_N")
        self.si_N.setMinimumSize(QSize(40, 0))
        self.si_N.setMinimum(1)
        self.si_N.setMaximum(999999999)

        self.horizontalLayout_2.addWidget(self.si_N)

        self.in_step = QLabel(WImportLinspace)
        self.in_step.setObjectName(u"in_step")

        self.horizontalLayout_2.addWidget(self.in_step)

        self.lf_step = FloatEdit(WImportLinspace)
        self.lf_step.setObjectName(u"lf_step")
        self.lf_step.setMinimumSize(QSize(40, 0))

        self.horizontalLayout_2.addWidget(self.lf_step)

        self.in_stop = QLabel(WImportLinspace)
        self.in_stop.setObjectName(u"in_stop")

        self.horizontalLayout_2.addWidget(self.in_stop)

        self.lf_stop = FloatEdit(WImportLinspace)
        self.lf_stop.setObjectName(u"lf_stop")
        self.lf_stop.setMinimumSize(QSize(40, 0))

        self.horizontalLayout_2.addWidget(self.lf_stop)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(WImportLinspace)

        QMetaObject.connectSlotsByName(WImportLinspace)

    # setupUi

    def retranslateUi(self, WImportLinspace):
        WImportLinspace.setWindowTitle(
            QCoreApplication.translate("WImportLinspace", u"Form", None)
        )
        self.c_type_lin.setItemText(
            0, QCoreApplication.translate("WImportLinspace", u"Start, End, N", None)
        )
        self.c_type_lin.setItemText(
            1, QCoreApplication.translate("WImportLinspace", u"Start, End, Step", None)
        )

        self.is_end.setText(
            QCoreApplication.translate("WImportLinspace", u"Include End Point", None)
        )
        self.in_start.setText(
            QCoreApplication.translate("WImportLinspace", u"Start: ", None)
        )
        self.in_N.setText(QCoreApplication.translate("WImportLinspace", u"N: ", None))
        self.in_step.setText(
            QCoreApplication.translate("WImportLinspace", u"Step: ", None)
        )
        self.in_stop.setText(
            QCoreApplication.translate("WImportLinspace", u"Stop: ", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

from numpy import cos, exp
from numpy import abs as np_abs


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the hole

    Parameters
    ----------
    self : HoleM54
        A HoleM54 object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the hole [m]

    """

    Rext = self.get_Rext()

    surf_list = self.build_geometry()
    point_list = list()
    for curve in surf_list[0].line_list:
        point_list.extend(curve.discretize())

    Rmax = max([np_abs(point) for point in point_list])
    Rmin = Rext - self.H0 - self.H1
    return (Rmin, Rmax)



# -*- coding: utf-8 -*-

from numpy import linspace


def get_data(self):
    """Generate the linspace vector

    Parameters
    ----------
    self : ImportGenVectLin
        An ImportGenVectLin object

    Returns
    -------
    vect: ndarray
        The generated linspace vector

    """

    vect = linspace(
        start=self.start, stop=self.stop, num=int(self.num), endpoint=self.endpoint
    )

    return self.edit_matrix(vect)



def get_simu_number(self):
    """Return the number of simulation defined by the Param Explorer

    Parameters
    ----------
    self : VarParam
        A VarParam object

    Returns
    -------
    nb_simu : int
        number of simulation
    """

    nb_simu = 1
    for paramexplorer in self.paramexplorer_list:
        nb_simu *= len(paramexplorer.value)

    return nb_simu



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/BoreUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/BoreUD
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Bore import Bore

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.BoreUD.get_bore_line import get_bore_line
except ImportError as error:
    get_bore_line = error


from ._check import InitUnKnowClassError
from .Line import Line


class BoreUD(Bore):
    """User Defined Bore shape"""

    VERSION = 1

    # cf Methods.Machine.BoreUD.get_bore_line
    if isinstance(get_bore_line, ImportError):
        get_bore_line = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use BoreUD method get_bore_line: " + str(get_bore_line)
                )
            )
        )
    else:
        get_bore_line = get_bore_line
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, line_list=-1, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "line_list" in list(init_dict.keys()):
                line_list = init_dict["line_list"]
        # Set the properties (value check and convertion are done in setter)
        self.line_list = line_list
        # Call Bore init
        super(BoreUD, self).__init__()
        # The class is frozen (in Bore init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        BoreUD_str = ""
        # Get the properties inherited from Bore
        BoreUD_str += super(BoreUD, self).__str__()
        if len(self.line_list) == 0:
            BoreUD_str += "line_list = []" + linesep
        for ii in range(len(self.line_list)):
            tmp = (
                self.line_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            BoreUD_str += "line_list[" + str(ii) + "] =" + tmp + linesep + linesep
        return BoreUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Bore
        if not super(BoreUD, self).__eq__(other):
            return False
        if other.line_list != self.line_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Bore
        diff_list.extend(super(BoreUD, self).compare(other, name=name))
        if (other.line_list is None and self.line_list is not None) or (
            other.line_list is not None and self.line_list is None
        ):
            diff_list.append(name + ".line_list None mismatch")
        elif self.line_list is None:
            pass
        elif len(other.line_list) != len(self.line_list):
            diff_list.append("len(" + name + ".line_list)")
        else:
            for ii in range(len(other.line_list)):
                diff_list.extend(
                    self.line_list[ii].compare(
                        other.line_list[ii], name=name + ".line_list[" + str(ii) + "]"
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Bore
        S += super(BoreUD, self).__sizeof__()
        if self.line_list is not None:
            for value in self.line_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Bore
        BoreUD_dict = super(BoreUD, self).as_dict(**kwargs)
        if self.line_list is None:
            BoreUD_dict["line_list"] = None
        else:
            BoreUD_dict["line_list"] = list()
            for obj in self.line_list:
                if obj is not None:
                    BoreUD_dict["line_list"].append(obj.as_dict(**kwargs))
                else:
                    BoreUD_dict["line_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        BoreUD_dict["__class__"] = "BoreUD"
        return BoreUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.line_list = None
        # Set to None the properties inherited from Bore
        super(BoreUD, self)._set_None()

    def _get_line_list(self):
        """getter of line_list"""
        if self._line_list is not None:
            for obj in self._line_list:
                if obj is not None:
                    obj.parent = self
        return self._line_list

    def _set_line_list(self, value):
        """setter of line_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "line_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("line_list", value, "[Line]")
        self._line_list = value

    line_list = property(
        fget=_get_line_list,
        fset=_set_line_list,
        doc=u"""List of line to draw the full bore

        :Type: [Line]
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QWidget

from ......Classes.HoleM58 import HoleM58
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM58.Gen_PHoleM58 import Gen_PHoleM58
from ......GUI.Dialog.DMatLib.MatLib import MatLib
from ......Methods.Slot.Slot import SlotCheckError


class PHoleM58(Gen_PHoleM58, QWidget):
    """Page to set the Hole Type 58"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for WHoleMag
    hole_name = "Slot Type 58"
    hole_type = HoleM58

    def __init__(self, hole=None, matlib=MatLib()):
        """Initialize the widget according to hole

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        hole : HoleM58
            current hole to edit
        matlib : MatLib
            Material Library
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.matlib = matlib
        self.hole = hole

        # Set FloatEdit unit
        self.lf_W0.unit = "m"
        self.lf_W1.unit = "m"
        self.lf_W2.unit = "m"
        self.lf_W3.unit = "rad"
        self.lf_R0.unit = "m"
        self.lf_H0.unit = "m"
        self.lf_H1.unit = "m"
        self.lf_H2.unit = "m"

        # Set default materials
        self.w_mat_0.setText("mat_void")
        self.w_mat_0.def_mat = "Air"
        self.w_mat_0.is_hide_button = True

        self.w_mat_1.setText("magnet_0")
        self.w_mat_1.def_mat = "Magnet1"
        self.w_mat_1.is_hide_button = True

        # Adapt GUI with/without magnet
        if hole.magnet_0 is None:  # SyRM
            self.img_slot.setPixmap(
                QPixmap(":/images/images/MachineSetup/SMHoleMag/HoleM58_no_mag.png")
            )
            self.W1 = 0
            self.W2 = 0
            self.w_mat_0.update(self.hole, "mat_void", self.matlib)
            self.w_mat_1.hide()
        else:
            # Set current material
            self.w_mat_0.update(self.hole, "mat_void", self.matlib)
            self.w_mat_1.update(self.hole.magnet_0, "mat_type", self.matlib)

        # Set unit name (m ou mm)
        self.u = gui_option.unit
        wid_list = [
            self.unit_W0,
            self.unit_W1,
            self.unit_W2,
            self.unit_R0,
            self.unit_H0,
            self.unit_H1,
            self.unit_H2,
        ]
        for wid in wid_list:
            wid.setText("[" + self.u.get_m_name() + "]")

        # Fill the fields with the machine values (if they're filled)
        self.lf_W0.setValue(self.hole.W0)
        self.lf_W1.setValue(self.hole.W1)
        self.lf_W2.setValue(self.hole.W2)
        self.lf_W3.setValue(self.hole.W3)
        self.lf_R0.setValue(self.hole.R0)
        self.lf_H0.setValue(self.hole.H0)
        self.lf_H1.setValue(self.hole.H1)
        self.lf_H2.setValue(self.hole.H2)

        # Display the main output of the hole (surface, height...)
        self.comp_output()

        # Connect the signal
        self.lf_W0.editingFinished.connect(self.set_W0)
        self.lf_W1.editingFinished.connect(self.set_W1)
        self.lf_W2.editingFinished.connect(self.set_W2)
        self.lf_W3.editingFinished.connect(self.set_W3)
        self.lf_R0.editingFinished.connect(self.set_R0)
        self.lf_H0.editingFinished.connect(self.set_H0)
        self.lf_H1.editingFinished.connect(self.set_H1)
        self.lf_H2.editingFinished.connect(self.set_H2)
        self.w_mat_0.saveNeeded.connect(self.emit_save)
        self.w_mat_1.saveNeeded.connect(self.emit_save)

    def set_W0(self):
        """Signal to update the value of W0 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.W0 = self.lf_W0.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_W1(self):
        """Signal to update the value of W1 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.W1 = self.lf_W1.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_W2(self):
        """Signal to update the value of W2 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.W2 = self.lf_W2.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_W3(self):
        """Signal to update the value of W3 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.W3 = self.lf_W3.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_R0(self):
        """Signal to update the value of R0 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.R0 = self.lf_R0.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H0(self):
        """Signal to update the value of H0 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.H0 = self.lf_H0.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H1(self):
        """Signal to update the value of H1 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.H1 = self.lf_H1.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_H2(self):
        """Signal to update the value of H2 according to the line edit

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        self.hole.H2 = self.lf_H2.value()
        self.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def comp_output(self):
        """Compute and display the hole output

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget
        """
        is_set = False
        if self.check() is None:
            try:
                # We compute the output only if the hole is correctly set
                # Compute all the needed output as string
                s_surf = format(self.u.get_m2(self.hole.comp_surface()), ".4g")
                m_surf = format(self.u.get_m2(self.hole.comp_surface_magnets()), ".4g")

                # Update the GUI to display the Output
                self.out_slot_surface.setText(
                    "Hole suface: " + s_surf + " " + self.u.get_m2_name()
                )
                self.out_magnet_surface.setText(
                    "Magnet surface: " + m_surf + " " + self.u.get_m2_name()
                )
                is_set = True
            except:
                pass

        if not is_set:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong)
            self.out_slot_surface.setText("Hole suface: ?")
            self.out_magnet_surface.setText("Magnet surface: ?")

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : PHoleM58
            A PHoleM58 widget

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        # Constraints and None
        try:
            self.hole.check()
        except SlotCheckError as error:
            return str(error)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()



# -*- coding: utf-8 -*-

from numpy import pi


def comp_volumes(self):
    """Compute the volumes of the Lamination

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    V_dict: dict
        Volume of the Lamination (Vlam, Vvent, Vyoke, Vteeth) [m**3]

    """

    Lf = self.comp_length()  # Include radial ventilation ducts

    S_dict = self.comp_surfaces()
    Vvent = Lf * S_dict["Svent"]
    # L1 is without ventilation ducts (volume to compute masses)
    Vlam = self.L1 * S_dict["Slam"]
    Vyoke = self.L1 * S_dict["Syoke"]
    Vteeth = self.L1 * S_dict["Steeth"]
    return {"Vlam": Vlam, "Vvent": Vvent, "Vyoke": Vyoke, "Vteeth": Vteeth}



# -*- coding: utf-8 -*-

from numpy import exp, pi, sqrt

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleM58
        A HoleM58 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the HoleM58

    """

    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"
    Rext = self.get_Rext()
    R0 = self.R0

    # Get all the points
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    Z10 = point_dict["Z10"]
    Z11 = point_dict["Z11"]
    Z12 = point_dict["Z12"]
    Zc1 = point_dict["Zc1"]
    Zc2 = point_dict["Zc2"]

    surf_list = list()

    surf_list = list()
    # Create all the surfaces for all the cases
    # Air surface Zc1 with magnet_0
    curve_list = list()
    if self.W2 > 0:
        curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Arc1(begin=Z3, end=Z4, radius=R0, is_trigo_direction=True))
    curve_list.append(Segment(Z4, Z5))
    if self.W2 > 0:
        curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z1))
    point_ref = Zc1
    S1 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Magnet_0 surface
    curve_list = list()
    curve_list.append(Segment(Z1, Z6))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z12))
    curve_list.append(Segment(Z12, Z1))
    point_ref = (Z1 + Z6 + Z7 + Z12) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T0_S0"
    S2 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air surface Zc2 with magnet_0
    curve_list = list()
    curve_list.append(Segment(Z12, Z7))
    if self.W2 + self.W1 < self.W0:
        curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Arc1(begin=Z9, end=Z10, radius=R0, is_trigo_direction=True))
    curve_list.append(Segment(Z10, Z11))
    if self.W2 + self.W1 < self.W0:
        curve_list.append(Segment(Z11, Z12))
    point_ref = Zc2
    S3 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Full surface, no magnet
    curve_list = list()
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Arc1(begin=Z3, end=Z4, radius=R0, is_trigo_direction=True))
    curve_list.append(Segment(Z4, Z5))
    curve_list.append(Segment(Z5, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Arc1(begin=Z9, end=Z10, radius=R0, is_trigo_direction=True))
    curve_list.append(Segment(Z10, Z11))
    curve_list.append(Segment(Z11, Z2))
    point_ref = Rext - self.H0 - self.H2 / 2
    S4 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Create the surface list by selecting the correct ones
    if self.magnet_0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        surf_list = [S1, S2, S3]
    else:
        S4.label = S1.label + "_R0_T0_S0"  # Hole
        surf_list = [S4]

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



from numpy import arcsin, exp


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW12
        A SlotW12 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """
    Rbo = self.get_Rbo()

    # alpha is the angle to rotate Z0 so ||Z1,Z8|| = 2*R2
    alpha = float(arcsin(self.R2 / Rbo))

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z1 = Z0 * exp(-1j * alpha)

    if self.is_outwards():
        Z2 = Z1 + self.H0
        Z3 = Z2 + self.R1 * 2
        Z4 = Z3 + self.H1
        Zc1 = Z2 + self.R1
        Zi2 = Z4.real + self.R2
    else:  # inward slot
        Z2 = Z1 - self.H0
        Z3 = Z2 - self.R1 * 2
        Z4 = Z3 - self.H1
        Zc1 = Z2 - self.R1
        Zi2 = Z4.real - self.R2
    Zc2 = Z4.real

    point_dict = dict()
    # symetry
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Zc1"] = Zc1
    point_dict["Zc2"] = Zc2
    point_dict["Z5"] = Z4.conjugate()
    point_dict["Z6"] = Z3.conjugate()
    point_dict["Z7"] = Z2.conjugate()
    point_dict["Z8"] = Z1.conjugate()
    point_dict["Zc3"] = Zc1.conjugate()
    point_dict["Zi1"] = Zc1 - 1j * self.R1
    point_dict["Zi2"] = Zi2
    point_dict["Zi3"] = point_dict["Zc3"] + 1j * self.R1

    return point_dict



# -*- coding: utf-8 -*-

# File generated according to DMachineSetup.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_DMachineSetup(object):
    def setupUi(self, DMachineSetup):
        if not DMachineSetup.objectName():
            DMachineSetup.setObjectName(u"DMachineSetup")
        DMachineSetup.resize(1001, 721)
        self.horizontalLayout_2 = QHBoxLayout(DMachineSetup)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.nav_step = QListWidget(DMachineSetup)
        self.nav_step.setObjectName(u"nav_step")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nav_step.sizePolicy().hasHeightForWidth())
        self.nav_step.setSizePolicy(sizePolicy)
        self.nav_step.setMaximumSize(QSize(190, 16777215))

        self.horizontalLayout_2.addWidget(self.nav_step)

        self.main_layout = QVBoxLayout()
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.b_save = QPushButton(DMachineSetup)
        self.b_save.setObjectName(u"b_save")

        self.horizontalLayout.addWidget(self.b_save)

        self.b_load = QPushButton(DMachineSetup)
        self.b_load.setObjectName(u"b_load")

        self.horizontalLayout.addWidget(self.b_load)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_step = QWidget(DMachineSetup)
        self.w_step.setObjectName(u"w_step")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.w_step.sizePolicy().hasHeightForWidth())
        self.w_step.setSizePolicy(sizePolicy1)
        self.w_step.setMinimumSize(QSize(800, 660))

        self.main_layout.addWidget(self.w_step)

        self.horizontalLayout_2.addLayout(self.main_layout)

        self.retranslateUi(DMachineSetup)

        QMetaObject.connectSlotsByName(DMachineSetup)

    # setupUi

    def retranslateUi(self, DMachineSetup):
        DMachineSetup.setWindowTitle(
            QCoreApplication.translate("DMachineSetup", u"Pyleecan Machine Setup", None)
        )
        self.b_save.setText(QCoreApplication.translate("DMachineSetup", u"Save", None))
        self.b_load.setText(QCoreApplication.translate("DMachineSetup", u"Load", None))

    # retranslateUi



import matplotlib.pyplot as plt
from numpy import pi, exp

from ....Classes.Arc1 import Arc1
from ....Classes.LamSlot import LamSlot
from ....Classes.Segment import Segment
from ....definitions import config_dict
from ....Functions.Plot import (
    ARROW_COLOR,
    ARROW_WIDTH,
    MAIN_LINE_COLOR,
    MAIN_LINE_STYLE,
    MAIN_LINE_WIDTH,
    P_FONT_SIZE,
    SC_FONT_SIZE,
    SC_LINE_COLOR,
    SC_LINE_STYLE,
    SC_LINE_WIDTH,
    TEXT_BOX,
    plot_quote,
)


def plot_schematics(
    self,
    is_default=False,
    is_add_schematics=True,
    is_add_main_line=True,
    save_path=None,
    is_show_fig=True,
):
    """Plot the schematics of the slot

    Parameters
    ----------
    self : CondType12
        A CondType12 object
    is_default : bool
        True: plot default schematics, else use current slot values
    is_add_schematics : bool
        True to display the schematics information (W0, H0...)
    is_add_main_line : bool
        True to display "main lines" (slot opening and 0x axis)
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    is_show_fig : bool
        To call show at the end of the method
    """

    # Use some default parameter
    if is_default:
        cond = type(self)(Wins_cond=40e-3, Wwire=10e-3, Wins_wire=2e-3, Nwppc=4)
        cond.plot_schematics(
            is_default=False,
            is_add_schematics=is_add_schematics,
            is_add_main_line=is_add_main_line,
            save_path=save_path,
            is_show_fig=is_show_fig,
        )
    else:
        # Getting the main plot
        self.plot(is_show_fig=False)  # center slot on Ox axis
        fig = plt.gcf()
        ax = plt.gca()
        a = self.Wwire / 2 + self.Wins_wire

        # Adding schematics
        if is_add_schematics:
            # Wwire
            line = Segment(a - self.Wwire / 2 + 1j * a, a + self.Wwire / 2 + 1j * a)
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="Wwire",
                offset_label=1j * self.Wins_wire,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # Wins_wire
            line = Segment(
                -self.Wwire / 2 - self.Wins_wire + 1j * self.Wins_wire,
                -self.Wwire / 2 - self.Wins_wire - 1j * self.Wins_wire,
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="2*Wins_wire",
                offset_label=-8 * self.Wins_wire,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # Wins_cond
            line = Segment(-1j * self.Wins_cond / 2, 1j * self.Wins_cond / 2)
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="Wins_cond",
                offset_label=-1j * self.Wins_cond * 0.25 + self.Wins_wire,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # Nwppc_rad
            ax.text(
                -a - self.Wwire,
                a,
                "Nwppc=4",
                fontsize=SC_FONT_SIZE,
                bbox=TEXT_BOX,
            )

        # Zooming and cleaning
        W = self.comp_width() * 1.05

        plt.axis("equal")
        ax.set_xlim(-W / 2, W / 2)
        ax.set_ylim(-W / 2, W / 2)
        fig.canvas.set_window_title(type(self).__name__ + " Schematics")
        ax.set_title("")
        ax.get_legend().remove()
        ax.set_axis_off()

        # Save / Show
        if save_path is not None:
            fig.savefig(save_path)
            plt.close()

        if is_show_fig:
            fig.show()



# -*- coding: utf-8 -*-


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW25
        A SlotW25 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    return self.H2



# -*- coding: utf-8 -*-
from ....Functions.Plot.plot_2D_Data import plot_2D_Data
from ....definitions import config_dict


def plot_mmf_unit(self, r_max=100, fig=None, is_show_fig=True):
    """Plot the winding unit mmf as a function of space
    Parameters
    ----------

    self : LamSlotWind
        an LamSlotWind object
    Na : int
        Space discretization
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    is_show_fig : bool
        To call show at the end of the method
    """

    name = ""
    if self.parent is not None and self.parent.name not in [None, ""]:
        name += self.parent.name + " "
    if self.is_stator:
        name += "Stator "
    else:
        name += "Rotor "

    # Compute the winding function and mmf
    wf = self.comp_wind_function(per_a=1)
    qs = self.winding.qs
    MMF_U, WF = self.comp_mmf_unit(Nt=1, Na=wf.shape[1])

    color_list = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][: qs + 1]
    plot_2D_Data(
        WF,
        "angle",
        "phase",
        data_list=[MMF_U],
        fig=fig,
        color_list=color_list,
        is_show_fig=is_show_fig,
        win_title=name + "Winding functions",
    )

    plot_2D_Data(
        WF,
        "wavenumber=[0," + str(r_max) + "]",
        "phase[0]",
        data_list=[MMF_U],
        fig=fig,
        color_list=color_list,
        is_show_fig=is_show_fig,
        win_title=name + "Winding function & MMF FFT",
    )



# -*- coding: utf-8 -*-


def get_pole_pair_number(self):
    """Return the number of pair of pole of the Lamination

    Parameters
    ----------
    self : LamSlotMulti
        A LamSlotMulti object

    Returns
    -------
    p: int
        Number of pair of pole

    """

    return self.get_Zs() // 2



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/SurfLine.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/SurfLine
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Surface import Surface

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.SurfLine.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Geometry.SurfLine.comp_length import comp_length
except ImportError as error:
    comp_length = error

try:
    from ..Methods.Geometry.SurfLine.comp_point_ref import comp_point_ref
except ImportError as error:
    comp_point_ref = error

try:
    from ..Methods.Geometry.SurfLine.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Geometry.SurfLine.discretize import discretize
except ImportError as error:
    discretize = error

try:
    from ..Methods.Geometry.SurfLine.get_lines import get_lines
except ImportError as error:
    get_lines = error

try:
    from ..Methods.Geometry.SurfLine.get_patches import get_patches
except ImportError as error:
    get_patches = error

try:
    from ..Methods.Geometry.SurfLine.plot_lines import plot_lines
except ImportError as error:
    plot_lines = error

try:
    from ..Methods.Geometry.SurfLine.rotate import rotate
except ImportError as error:
    rotate = error

try:
    from ..Methods.Geometry.SurfLine.scale import scale
except ImportError as error:
    scale = error

try:
    from ..Methods.Geometry.SurfLine.translate import translate
except ImportError as error:
    translate = error


from ._check import InitUnKnowClassError
from .Line import Line


class SurfLine(Surface):
    """SurfLine defined by list of lines that delimit it, label and point reference."""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.SurfLine.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Geometry.SurfLine.comp_length
    if isinstance(comp_length, ImportError):
        comp_length = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfLine method comp_length: " + str(comp_length)
                )
            )
        )
    else:
        comp_length = comp_length
    # cf Methods.Geometry.SurfLine.comp_point_ref
    if isinstance(comp_point_ref, ImportError):
        comp_point_ref = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfLine method comp_point_ref: " + str(comp_point_ref)
                )
            )
        )
    else:
        comp_point_ref = comp_point_ref
    # cf Methods.Geometry.SurfLine.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfLine method comp_surface: " + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Geometry.SurfLine.discretize
    if isinstance(discretize, ImportError):
        discretize = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method discretize: " + str(discretize))
            )
        )
    else:
        discretize = discretize
    # cf Methods.Geometry.SurfLine.get_lines
    if isinstance(get_lines, ImportError):
        get_lines = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method get_lines: " + str(get_lines))
            )
        )
    else:
        get_lines = get_lines
    # cf Methods.Geometry.SurfLine.get_patches
    if isinstance(get_patches, ImportError):
        get_patches = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SurfLine method get_patches: " + str(get_patches)
                )
            )
        )
    else:
        get_patches = get_patches
    # cf Methods.Geometry.SurfLine.plot_lines
    if isinstance(plot_lines, ImportError):
        plot_lines = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method plot_lines: " + str(plot_lines))
            )
        )
    else:
        plot_lines = plot_lines
    # cf Methods.Geometry.SurfLine.rotate
    if isinstance(rotate, ImportError):
        rotate = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method rotate: " + str(rotate))
            )
        )
    else:
        rotate = rotate
    # cf Methods.Geometry.SurfLine.scale
    if isinstance(scale, ImportError):
        scale = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method scale: " + str(scale))
            )
        )
    else:
        scale = scale
    # cf Methods.Geometry.SurfLine.translate
    if isinstance(translate, ImportError):
        translate = property(
            fget=lambda x: raise_(
                ImportError("Can't use SurfLine method translate: " + str(translate))
            )
        )
    else:
        translate = translate
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, line_list=-1, point_ref=0, label="", init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "line_list" in list(init_dict.keys()):
                line_list = init_dict["line_list"]
            if "point_ref" in list(init_dict.keys()):
                point_ref = init_dict["point_ref"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        self.line_list = line_list
        # Call Surface init
        super(SurfLine, self).__init__(point_ref=point_ref, label=label)
        # The class is frozen (in Surface init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SurfLine_str = ""
        # Get the properties inherited from Surface
        SurfLine_str += super(SurfLine, self).__str__()
        if len(self.line_list) == 0:
            SurfLine_str += "line_list = []" + linesep
        for ii in range(len(self.line_list)):
            tmp = (
                self.line_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            SurfLine_str += "line_list[" + str(ii) + "] =" + tmp + linesep + linesep
        return SurfLine_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Surface
        if not super(SurfLine, self).__eq__(other):
            return False
        if other.line_list != self.line_list:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Surface
        diff_list.extend(super(SurfLine, self).compare(other, name=name))
        if (other.line_list is None and self.line_list is not None) or (
            other.line_list is not None and self.line_list is None
        ):
            diff_list.append(name + ".line_list None mismatch")
        elif self.line_list is None:
            pass
        elif len(other.line_list) != len(self.line_list):
            diff_list.append("len(" + name + ".line_list)")
        else:
            for ii in range(len(other.line_list)):
                diff_list.extend(
                    self.line_list[ii].compare(
                        other.line_list[ii], name=name + ".line_list[" + str(ii) + "]"
                    )
                )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Surface
        S += super(SurfLine, self).__sizeof__()
        if self.line_list is not None:
            for value in self.line_list:
                S += getsizeof(value)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Surface
        SurfLine_dict = super(SurfLine, self).as_dict(**kwargs)
        if self.line_list is None:
            SurfLine_dict["line_list"] = None
        else:
            SurfLine_dict["line_list"] = list()
            for obj in self.line_list:
                if obj is not None:
                    SurfLine_dict["line_list"].append(obj.as_dict(**kwargs))
                else:
                    SurfLine_dict["line_list"].append(None)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SurfLine_dict["__class__"] = "SurfLine"
        return SurfLine_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.line_list = None
        # Set to None the properties inherited from Surface
        super(SurfLine, self)._set_None()

    def _get_line_list(self):
        """getter of line_list"""
        if self._line_list is not None:
            for obj in self._line_list:
                if obj is not None:
                    obj.parent = self
        return self._line_list

    def _set_line_list(self, value):
        """setter of line_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "line_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("line_list", value, "[Line]")
        self._line_list = value

    line_list = property(
        fget=_get_line_list,
        fset=_set_line_list,
        doc=u"""List of Lines 

        :Type: [Line]
        """,
    )



# -*- coding: utf-8 -*-
from numpy import array, mod, zeros

from ....Methods.Machine.WindingCW2LT import WindingT1DefMsError
from ....Methods.Machine.Winding import WindingError
from ....Functions.Winding.reverse_wind_mat import reverse_wind_mat
from ....Functions.Winding.shift_wind_mat import shift_wind_mat


def comp_connection_mat(self, Zs=None):
    """Compute the Winding Matrix (for winding type 1)
    type 1 : TOOTH WINDING, DOUBLE LAYER ALL TEETH WOUND, ORTHORADIAL
    SUPERPOSITION  (Nlay_rad=1,Nlay_tan=2)

    Parameters
    ----------
    self : WindingCW2LT
        A: WindingCW2LT object
    Zs : int
        Number of Slot (Integer >0)

    Returns
    -------
    wind_mat: numpy.ndarray
        Winding Matrix (Nlay_rad, Nlay_tan, Zs, qs)

    Raises
    ------
    WindingT1DefMsError
        You must have 0.25< Zs/2/p/qs <= 0.5

    """
    if Zs is None:
        if self.parent is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object."
            )

        if self.parent.slot is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object with Slot."
            )

        Zs = self.parent.slot.Zs

    assert Zs > 0, "Zs must be >0"
    assert Zs % 1 == 0, "Zs must be an integer"

    # ex "Effect of Pole and Slot Combination on Noise and Vibration in Permanent
    # Magnet Synchronous Motor" creates highest harmonic at Zs/2+1 and Zs/2-1
    nlay = 2.0
    qs = float(self.qs)
    p = float(self.p)
    Ncgr = Zs / nlay / qs
    ms = Zs / 2.0 / p / qs
    # Ncspc= Zs/(2.0*qs*self.Npcpp/nlay)  # number of coils in series per parallel circuit
    # Ntspc = self.Ntcoil * Ncspc #Number of turns in series per phase
    Ntcoil = self.Ntcoil  # number of turns per coils

    wind_mat = zeros((1, 2, Zs, int(qs)))
    # creates highest harmonic at Zs/2+1 and Zs/2-1
    if ms == 0.5:  # then Zs/qs is integer
        # traditional non overlapping all teeth wound winding
        for q in range(0, int(qs)):
            for k in range(0, int(Zs / qs)):  # number of Ncgr coils
                xenc = q + array([1, 0]) + k * qs
                wind_mat[0, 0, int(mod(xenc[0] - 1, Zs)), q] = +Ntcoil  # right/top/2
                wind_mat[0, 1, int(mod(xenc[1] - 1, Zs)), q] = -Ntcoil  # left/bottom/1
    elif ms != 0.5 and Ncgr % 1 == 0:
        # ms!=0.5 and Ncgr is an integer (ms>0.25 && ms<0.5)
        # new algorithm to reverse the coils
        for q in range(0, int(qs)):
            for k in range(0, int(nlay)):  # number of Ncgr coils
                for l in range(0, int(Ncgr)):
                    id0 = int(mod(q * Ncgr + 0 + k * qs * Ncgr - 1 + l, Zs))
                    id1 = int(mod(q * Ncgr + 1 + k * qs * Ncgr - 1 + l, Zs))
                    # left / bottom / 1
                    wind_mat[0, 1, id0, q] = -((-1) ** (l + q - 1 + k)) * Ntcoil
                    # right / top / 2
                    wind_mat[0, 0, id1, q] = +((-1) ** (l + q - 1 + k)) * Ntcoil

    else:
        raise WindingT1DefMsError(
            "Winding geometry not handled yet. Enter "
            "your own winding matrix with WindingUD"
            " and contact EOMYS"
        )

    # Set default values
    if self.is_reverse_wind is None:
        self.is_reverse_wind = False
    if self.Nslot_shift_wind is None:
        self.Nslot_shift_wind = 0
    # Apply the transformations
    if self.is_reverse_wind:
        wind_mat = reverse_wind_mat(wind_mat)
    if self.Nslot_shift_wind > 0:
        wind_mat = shift_wind_mat(wind_mat, self.Nslot_shift_wind)

    return wind_mat



# -*- coding: utf-8 -*-


def comp_radius_mid_yoke(self):
    """Compute the Lamination middle of the yoke radius

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    Ry: float
        middle of the yoke radius [m]

    """

    return (self.Rext + self.Rint) / 2



# -*- coding: utf-8 -*-

from numpy import pi, cos, sin, tan, arcsin, exp


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the slot

    Parameters
    ----------
    self : HoleM57
        A HoleM57 object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the slot [m]
    """
    Rext = self.get_Rext()

    Rmax = Rext - self.H1

    # "Tooth" angle (P1',0,P1)
    alpha_T = 2 * arcsin(self.W3 / (2 * (Rext - self.H1)))
    # magnet pole pitch angle (Z1,0,Z1')
    alpha_S = 2 * pi / self.Zh - alpha_T
    # Angle (P1,P1',P4') and (P5',P4', )
    alpha = (pi - self.W0) / 2

    Z1 = (Rext - self.H1) * exp(-1j * alpha_S / 2)
    x11 = 2 * sin(alpha_S / 2) * (Rext - self.H1)  # Distance from P1 to P1'
    # In rect triangle P4, P1, perp (P1,P1') with P4
    H = tan(alpha) * (x11 / 2 - self.W1 / 2)
    Z4 = Z1.real - H - 1 * self.W1 / 2
    x45 = self.H2 / cos(alpha)  # distance from P4 to P5
    Z5 = Z4 - x45

    Rmin = abs(Z5)

    return (Rmin, Rmax)



def split_half(self, is_begin=True):
    """Cut the line in half (modify the object)

    Parameters
    ----------
    self : Arc2
        An Arc2 object
    is_begin : bool
        True to keep the part begin=>middle, False for the part middle=>end

    Returns
    -------
    """

    if not is_begin:
        self.begin = self.get_middle()
    self.angle = self.angle / 2



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface(self):
    """Compute the surface of the Frame

    Parameters
    ----------
    self : Frame
        A Frame object

    Returns
    -------
    Sfra: float
        Surface of the Frame [m**2]

    """

    # Surface of the external disk
    S_ext = (self.Rext ** 2) * pi
    # Surface of the internal disk
    S_int = (self.Rint ** 2) * pi

    return S_ext - S_int



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW26 import *


def check(self):
    """Check that the SlotW26 object is correct

    Parameters
    ----------
    self : SlotW26
        A SlotW26 object

    Returns
    -------
    None

    Raises
    -------
    S26_WCheckError
        You must have W0 < 2*R1
    """
    if self.R1 * 2 <= self.W0:
        raise S26_WCheckError("You must have W0 < 2*R1")



from ....Methods.Slot.Slot import SlotCheckError


def check(self):
    """Check that the HoleUD is correctly defined"""
    if self.surf_list is None:
        self.surf_list = list()
    if self.magnet_dict is None:
        self.magnet_dict = dict()
    if len(self.surf_list) == 0:
        raise SlotCheckError("HoleUD must have at least one surface")
    for key in self.magnet_dict:
        if key[:7] != "magnet_":
            raise SlotCheckError(
                "HoleUD magnet_dict must use key like 'magnet_X' with X the index of the magnet. Found: "
                + key
            )



# -*- coding: utf-8 -*-

from numpy import pi
from pyleecan.Classes.Segment import Segment


def comp_magnetization_dict(self, is_north=True):
    """Compute the dictionary of the magnetization direction of the magnets (key=magnet_X, value=angle[rad])
    Mangetization angle with Hole centered on Ox axis

    Parameters
    ----------
    self : HoleM50
        a HoleM50 object
    is_north: True
        True: comp north magnetization, else add pi [rad]

    Returns
    -------
    mag_dict: dict
        magnetization dictionary (key=magnet_X, value=angle[rad])
    """

    # Comp magnet
    point_dict = self._comp_point_coordinate()

    mag_dict = dict()
    S0 = Segment(point_dict["Z5"], point_dict["Z4"])
    mag_dict["magnet_0"] = S0.comp_normal()
    S1 = Segment(point_dict["Z4s"], point_dict["Z5s"])
    mag_dict["magnet_1"] = S1.comp_normal()

    if self.magnetization_dict_offset is not None:
        for key, value in self.magnetization_dict_offset:
            mag_dict[key] += value

    if not is_north:
        mag_dict["magnet_0"] += pi
        mag_dict["magnet_1"] += pi

    return mag_dict



# -*- coding: utf-8 -*-

from numpy import arcsin


def comp_angle_opening(self):
    """Compute the slot opening angle of the Slot

    Parameters
    ----------
    self : Slot19
        A Slot19 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    Rbo = self.get_Rbo()

    if self.Wx_is_rad:
        return self.W0
    else:
        return float(2 * arcsin(self.W0 / (2 * Rbo)))



# -*- coding: utf-8 -*-

from numpy import sin, pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()

    # Wint is the with at the top of the opening
    alpha = self.comp_angle_opening()
    if self.is_outwards():
        Wint = 2 * sin(alpha / 2) * (Rbo + self.H0)
        # Arc between opening H0 and wedge H1
        Sarc = ((Rbo + self.H0) ** 2.0) / 2.0 * (alpha - sin(alpha))
        # The opening part is radial
        S1 = (pi * ((Rbo + self.H0) ** 2) - pi * (Rbo ** 2)) * alpha / (2 * pi) - Sarc
    else:
        Wint = 2 * sin(alpha / 2) * (Rbo - self.H0)
        # Arc between opening H0 and wedge H1
        Sarc = ((Rbo - self.H0) ** 2.0) / 2.0 * (alpha - sin(alpha))
        # The opening part is radial
        S1 = (pi * (Rbo ** 2) - pi * ((Rbo - self.H0) ** 2)) * alpha / (2 * pi) + Sarc

    S2 = 0.5 * (Wint + self.W1) * self.H1
    Swind = self.comp_surface_active()

    return S1 + S2 + Swind



# -*- coding: utf-8 -*-


def has_magnet(self):
    """Return if the Hole has magnets

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    has_magnet : bool
        False

    """

    return False



# -*- coding: utf-8 -*-
from ....Methods.Slot.VentilationTrap import *


def check(self):
    """Make sure that the ventialtion parameters are correctly set

    Parameters
    ----------
    self : VentilationTrap
        A VentilationTrap object
    """

    if not isinstance(self.Zh, int):
        raise VentilationTrapInstanceError("Zh must be a integer")
    if not isinstance(self.H0, float) and not isinstance(self.H0, int):
        raise VentilationTrapInstanceError("H0 must be a float or int")
    if not isinstance(self.D0, float) and not isinstance(self.D0, int):
        raise VentilationTrapInstanceError("D0 must be a float or int")
    if not isinstance(self.Alpha0, float) and not isinstance(self.Alpha0, int):
        raise VentilationTrapInstanceError("Alpha0 must be a float or int")



from ...Classes.Segment import Segment
from ...Classes.Arc1 import Arc1
from ...Classes.Arc2 import Arc2
from math import cos, sin, pi
from logging import getLogger
from ...loggers import GUI_LOG_NAME


def dxf_to_pyleecan_list(entities):
    """
    Create the pyleecan object corresponding to the DXF entities
    Currently handles Line and Arc
    """
    obj_list = []
    for entity in entities:
        dxftype = entity.dxftype()
        dxf = entity.dxf
        if dxftype == "LINE":  # {"LINE", "XLINE", "RAY"}:
            start = complex(*dxf.start[:-1])
            end = complex(*dxf.end[:-1])
            obj_list.append(Segment(start, end))
        elif dxftype == "ARC":
            start_angle = dxf.start_angle / 180 * pi
            end_angle = dxf.end_angle / 180 * pi
            if start_angle > pi:
                start_angle -= 2 * pi
            if end_angle > pi:
                end_angle -= 2 * pi
            center = complex(*dxf.center[:-1])
            begin = (
                dxf.center[0]
                + dxf.radius * cos(start_angle)
                + 1j * (dxf.center[1] + dxf.radius * sin(start_angle))
            )
            end = (
                dxf.center[0]
                + dxf.radius * cos(end_angle)
                + 1j * (dxf.center[1] + dxf.radius * sin(end_angle))
            )
            obj_list.append(
                Arc1(begin=begin, end=end, radius=dxf.radius, is_trigo_direction=True)
            )
        else:
            getLogger(GUI_LOG_NAME).warning(
                "Unable to load dxftype="
                + str(dxftype)
                + ". Removing element from preview"
            )

    return obj_list



# -*- coding: utf-8 -*-

from ....Classes.MeshMat import MeshMat


def plot_mesh(
    self,
    label=None,
    index=None,
    indices=None,
    save_path=None,
    group_names=None,
    is_show_fig=True,
):
    """Plot the mesh using pyvista plotter.

    Parameters
    ----------
    self : MeshSolution
        a MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of the points to extract (optional)
    is_show_fig : bool
        To call show at the end of the method
    Returns
    -------
    """
    if group_names is not None:
        meshsol_grp = self.get_group(group_names)
        meshsol_grp.plot_mesh(label, index, indices, save_path, None)
    else:

        if save_path is None:
            try:
                import pyvistaqt as pv

                is_pyvistaqt = True
            except:
                import pyvista as pv

                is_pyvistaqt = False
        else:
            import pyvista as pv

            is_pyvistaqt = False

        print(save_path)

        # Get the mesh
        mesh_obj = self.get_mesh(label=label, index=index)
        if isinstance(mesh_obj, MeshMat):
            mesh = mesh_obj.get_mesh_pv(indices=indices)
        else:
            mesh = mesh_obj.get_mesh_pv(indices=indices)

        # Configure plot
        if is_pyvistaqt:
            p = pv.BackgroundPlotter()
            p.set_background("white")
        else:
            pv.set_plot_theme("document")
            p = pv.Plotter(notebook=False)
        p.add_mesh(
            mesh,
            color="grey",
            opacity=1,
            show_edges=True,
            edge_color="white",
            line_width=1,
        )
        if self.dimension == 2:
            p.view_xy()
        if save_path is None and is_show_fig:
            p.show()
        elif save_path is not None and is_show_fig:
            p.show(interactive=False, screenshot=save_path)



from ....Classes.Machine import Machine


def comp_desc_dict(self):
    """Compute a dictionnary with the main parameters/output of the machine

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    desc_dict: list
        list of dictionnary containing the main parameters of the machine
    """

    desc_dict = Machine.comp_desc_dict(self)

    Zr_dict = dict(
        {
            "name": "Zr",
            "path": "machine.rotor.slot.Zs",
            "verbose": "Rotor slot number",
            "type": int,
            "unit": "",
            "is_input": True,
            "value": self.rotor.slot.Zs,
        }
    )
    desc_dict.insert(2, Zr_dict)
    return desc_dict



# -*- coding: utf-8 -*-
from ....Classes.Hole import Hole


def comp_radius_mid_yoke(self):
    """Compute the Lamination middle of the yoke radius

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    Ry: float
        middle of the yoke radius [m]

    """

    Rmin = self.Rext
    Rmax = self.Rint
    for hole in self.hole:
        if type(hole) is not Hole:
            (Rmin_hole, Rmax_hole) = hole.comp_radius()
            Rmin = min(Rmin, Rmin_hole)
            Rmax = max(Rmax, Rmax_hole)
    if self.is_internal:
        return (self.Rint + Rmin) / 2
    else:
        return (self.Rext + Rmax) / 2



# -*- coding: utf-8 -*-

# File generated according to SPreview.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Dialog.DMachineSetup.SPreview.WMachineTable.WMachineTable import (
    WMachineTable,
)
from .....GUI.Tools.MPLCanvas import MPLCanvas2

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SPreview(object):
    def setupUi(self, SPreview):
        if not SPreview.objectName():
            SPreview.setObjectName(u"SPreview")
        SPreview.resize(532, 450)
        SPreview.setMinimumSize(QSize(0, 0))
        self.verticalLayout = QVBoxLayout(SPreview)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.w_plot = MPLCanvas2(SPreview)
        self.w_plot.setObjectName(u"w_plot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_plot.sizePolicy().hasHeightForWidth())
        self.w_plot.setSizePolicy(sizePolicy)
        self.w_plot.setMinimumSize(QSize(300, 300))
        self.w_plot.setMaximumSize(QSize(16777215, 16777215))

        self.horizontalLayout_2.addWidget(self.w_plot)

        self.tab_machine = WMachineTable(SPreview)
        self.tab_machine.setObjectName(u"tab_machine")

        self.horizontalLayout_2.addWidget(self.tab_machine)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.b_previous = QPushButton(SPreview)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout.addWidget(self.b_previous)

        self.b_next = QPushButton(SPreview)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout.addWidget(self.b_next)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(SPreview)

        QMetaObject.connectSlotsByName(SPreview)

    # setupUi

    def retranslateUi(self, SPreview):
        SPreview.setWindowTitle(QCoreApplication.translate("SPreview", u"Form", None))
        self.b_previous.setText(
            QCoreApplication.translate("SPreview", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SPreview", u"Next", None))

    # retranslateUi



from logging import getLogger
from ..loggers import DEFAULT_LOG_NAME


def get_logger(obj):
    """Get the object logger or its parent's one

    Parameters
    ----------
    obj :
        A pyleecan object

    Returns
    -------
    logger : logging.Logger
        Pyleecan object dedicated logger
    """

    if hasattr(obj, "logger_name"):  # Object logger
        return getLogger(obj.logger_name)
    elif obj.parent != None:  # Parent logger
        return obj.parent.get_logger()
    else:  # Default logger
        return getLogger(DEFAULT_LOG_NAME)



# -*- coding: utf-8 -*-
from ....Functions.MeshSolution.get_area import get_area
from ....Functions.MeshSolution.get_indices_limited import get_indices_limited


def get_demag(self, Hmax, group_name=None):
    """Get the surface area of the magnets that exceed a given field strenght H

    Parameters
    ----------
    Hmax : float
        demagnetization field strenght

    group_name : str
        optional name of the group, default group is 'rotor magnets'

    Return
    ------
    area : float
        surface area that exceed the max. field strength

    area_ref : float
        total magnets surface area as a reference

    indices : list
        list of cells indices that exceed the max. field strength

    """
    if group_name is None:
        group_name = "rotor magnets"
    if group_name in self.meshsolution.group.keys():
        inds = get_indices_limited(
            self.meshsolution, label="H", group_names=group_name, max_value=Hmax
        )

        area_ref = get_area(self.meshsolution, group_names=group_name)
        area = get_area(self.meshsolution, group_names=group_name, indices=inds)
    else:
        area, area_ref, inds = None, None, []

    return area, area_ref, inds



# -*- coding: utf-8 -*-

from ...Classes.LamSlot import LamSlot
from ...Classes.LamHole import LamHole
from ...Classes.HoleM50 import HoleM50
from ...Classes.HoleM51 import HoleM51
from ...Classes.HoleM52 import HoleM52
from ...Classes.HoleM53 import HoleM53
from ...Classes.LamSlotMag import LamSlotMag
from ...Functions.FEMM import acsolver, pbtype, precision, minangle
from ...Functions.FEMM import FEMM_GROUPS


def comp_FEMM_dict(machine, Kgeo_fineness, Kmesh_fineness, type_calc_leakage=0):
    """Compute the parameters needed for FEMM simulations

    Parameters
    ----------
    machine : Machine
        The machine to draw
    Kgeo_fineness : float
        global coefficient to adjust geometry fineness in FEMM
        (1 : default ; > 1 : finner ; < 1 : less fine)
    Kmesh_fineness : float
        global coefficient to adjust mesh fineness in FEMM
        (1 :default ; > 1 : finner ; < 1 : less fine)
    type_calc_leakage : int
        0 no leakage calculation
        1 calculation using single slot

    Returns
    -------
    FEMM_dict : dict
        Dictionnary containing the main parameters of FEMM

    """

    # Recompute because machine may has been modified
    Hsy = machine.stator.comp_height_yoke()
    Hstot = machine.stator.Rext - machine.stator.Rint - Hsy  # Works with holes and slot
    Wgap_mec = machine.comp_width_airgap_mec()
    Hry = machine.rotor.comp_height_yoke()
    Hrtot = machine.rotor.Rext - machine.rotor.Rint - Hry  # Works with holes and slot

    FEMM_dict = dict()
    FEMM_dict["is_close_model"] = 0
    FEMM_dict["acsolver"] = acsolver
    FEMM_dict["pbtype"] = pbtype
    FEMM_dict["precision"] = precision
    FEMM_dict["minangle"] = minangle
    FEMM_dict["freqpb"] = 0  # setting 2D magnetostatic problem

    FEMM_dict["smart_mesh"] = 0
    FEMM_dict["automesh"] = 0  # 1 to let the solver define the mesh in all
    # regions(except in the airgap), otherwise meshsize_XXX parameters are used
    FEMM_dict["automesh_airgap"] = 0  # 1 to let the solver define the mesh in the
    #  airgap, otherwise meshsize_airgap is used
    FEMM_dict["automesh_segments"] = 0  # 1 to let the solver define the mesh
    # points along arcs and segments, otherwise arcspan_XXX and
    # elementsize_XXX are used

    FEMM_dict["maxsegdeg"] = 1  # max angular width of elementary segment along
    # arc discretization(default: 1)
    FEMM_dict["maxelementsize"] = 1  # max length of elementary segment along
    # segment discretization(1 for automatic meshing) "elementsize" in FEMM doc
    FEMM_dict["arcspan"] = 1 / Kgeo_fineness  # max span of arc element in degrees

    FEMM_dict["Lfemm"] = (
        machine.stator.comp_length() + machine.rotor.comp_length()
    ) / 2

    # stator slot region mesh and segments max element size parameter
    # If Hstot = 0 there is no slot and the region parameter won't be used
    FEMM_dict["meshsize_slotS"] = Hstot / 8 / Kmesh_fineness
    FEMM_dict["elementsize_slotS"] = Hstot / 8 / Kmesh_fineness

    # stator yoke region mesh and segments max element size parameter
    FEMM_dict["meshsize_yokeS"] = Hsy / 4 / Kmesh_fineness
    FEMM_dict["elementsize_yokeS"] = Hsy / 4 / Kmesh_fineness

    # rotor slot region mesh and segments max element size parameter
    if type(machine.rotor) == LamSlotMag or type(machine.rotor) == LamHole:
        FEMM_dict["meshsize_slotR"] = Hry / 4 / Kmesh_fineness
        FEMM_dict["elementsize_slotR"] = Hry / 4 / Kmesh_fineness
    else:
        FEMM_dict["meshsize_slotR"] = Hrtot / 8 / Kmesh_fineness
        FEMM_dict["elementsize_slotR"] = Hrtot / 8 / Kmesh_fineness

    # rotor yoke region mesh and segments max element size parameter
    FEMM_dict["meshsize_yokeR"] = Hry / 4 / Kmesh_fineness
    FEMM_dict["elementsize_yokeR"] = Hry / 4 / Kmesh_fineness

    # airgap region mesh and segments max element size parameter
    FEMM_dict["meshsize_airgap"] = Wgap_mec / 3 / Kmesh_fineness
    FEMM_dict["elementsize_airgap"] = Wgap_mec / 3 / Kmesh_fineness

    # stator magnet region mesh and segments max element size parameter
    if type(machine.stator) == LamSlotMag:
        Hmag = machine.stator.slot.comp_height_active()
        FEMM_dict["meshsize_magnetS"] = Hmag / 4 / Kmesh_fineness
        FEMM_dict["elementsize_magnetS"] = Hmag / 4 / Kmesh_fineness
    else:
        FEMM_dict["meshsize_magnetS"] = None

    # rotor magnet region mesh and segments max element size parameter
    if type(machine.rotor) == LamSlotMag:
        Hmag = machine.rotor.slot.comp_height_active()
        FEMM_dict["meshsize_magnetR"] = Hmag / 4 / Kmesh_fineness
        FEMM_dict["elementsize_magnetR"] = Hmag / 4 / Kmesh_fineness
    elif type(machine.rotor) == LamHole:
        Hmag = machine.rotor.hole[0].comp_height()
        FEMM_dict["meshsize_magnetR"] = Hmag / 4 / Kmesh_fineness
        FEMM_dict["elementsize_magnetR"] = Hmag / 4 / Kmesh_fineness
    else:
        FEMM_dict["meshsize_magnetR"] = None

    # mesh parameter for basic air regions (ventilation ducts etc)
    FEMM_dict["meshsize_air"] = Hry / 4 / Kmesh_fineness
    FEMM_dict["meshsize_wedge"] = Hstot / 20 / Kmesh_fineness

    # mesh parameter for stator and rotor slot region
    if type_calc_leakage == 1:
        FEMM_dict["is_close_model"] = 1
        FEMM_dict["meshsize_slotS"] = Hstot / 50
        FEMM_dict["meshsize_slotR"] = Hrtot / 50

    # Set groups
    FEMM_dict["groups"] = dict()
    for grp in FEMM_GROUPS:
        FEMM_dict["groups"][grp] = FEMM_GROUPS[grp]["ID"]

    return FEMM_dict



from ....Methods import ParentMissingError


def get_is_stator(self):
    """Return True if the parent lamination is stator and False if is a rotor

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    is_stator: bool
        True if the Lamination is a stator and False if not

    """

    if self.parent is not None:
        return self.parent.is_stator
    else:
        raise ParentMissingError("Error: The slot is not inside a Lamination")



from numpy import angle, exp


def comp_normal(self):
    """Compute the normal direction of the Line
    Normal point is "on top" (begin=O and end on Ox)

    Parameters
    ----------
    self : Line
        a Line object

    Returns
    -------
    normal: float
        Angle of the vector between (Zbegin+Zend)/2 (even for arc) and the normal point [rad]
    """

    Zb = self.get_begin()
    Ze = self.get_end()
    Zm = (Zb + Ze) / 2  # "Middle" of the line
    L = self.comp_length() / 3

    # In ref begin=O and end on Ox
    Zm2 = (Zm - Zb) * exp(-1j * angle(Ze - Zb))
    Zn2 = Zm2 + 1j * L

    Zn = Zn2 * exp(1j * angle(Ze - Zb)) + Zb

    return angle(Zn - Zm)



# -*- coding: utf-8 -*-

import numpy as np


def get_coord(self, point_tags):
    """Return the coordinates of point(s).

    Parameters
    ----------
    self : PointMat
        an PointMat object
    point_tags : np.array
        an array of point tags

    Returns
    -------
    coord: np.array
        an array of point coordinates

    """

    nd_case = np.size(point_tags)
    coord = np.zeros((nd_case, 2))
    if nd_case == 1:
        coord = self.coordinate[point_tags, :]
    else:
        coord = self.coordinate[point_tags, :]

    if np.size(coord) == 0:
        return None
    else:
        return coord



from ....Methods.Slot.Slot import SlotCheckError


class S58_NoneError(SlotCheckError):
    """Raised when a propery of HoleM58 is None"""

    pass


class S58_WCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import abs as np_abs, angle as np_angle, exp, pi


def get_middle(self):
    """Return the point at the middle of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    Zmid: complex
        Complex coordinates of the middle of the Arc3
    """

    # We use the complex representation of the point
    z1 = self.begin
    zc = self.get_center()
    R = self.comp_radius()

    # Generation of the point by rotation
    if self.is_trigo_direction:  # Top
        Zmid = R * exp(1j * pi / 2.0)
    else:  # Bottom
        Zmid = R * exp(-1j * pi / 2.0)

    # Geometric transformation : return to the main axis
    Zmid = Zmid * exp(1j * np_angle(z1 - zc)) + zc

    # Return (0,0) if the point is too close from 0
    if np_abs(Zmid) < 1e-6:
        Zmid = 0

    return Zmid



# -*- coding: utf-8 -*-
from os.path import join

from ....Classes.ElmerResultsVTU import ElmerResultsVTU

# TODO define "Results" dir, etc. in __init__


def get_meshsolution(self, output):
    """Get and set the mesh data and solution data.

    Parameters
    ----------
    self : StructElmer
        a StructElmer object
    save_path: str
        Full path to folder in which the results are saved

    Returns
    -------
    success: bool
        Information if meshsolution could be created

    """
    # logger
    logger = self.get_logger()

    fea_path = self.get_path_save_fea(output)

    # if meshsoltion is not requested set meshsolution to None
    if not self.is_get_mesh or not self.is_save_FEA:
        logger.info("StructElmer: MeshSolution is not stored by request.")
        output.struct.meshsolution = None
        return False

    # setup Elmer result helper class
    ElmerVtu = ElmerResultsVTU()

    ElmerVtu.store_dict = {
        "displacement": {
            "name": "Displacement",
            "unit": "mm",
            "symbol": "disp",
            "norm": 1e-3,
        },
        "vonmises": {
            "name": "Von Mises Stress",
            "unit": "MPa",
            "symbol": "vonmises",
            "norm": 1e6,
        },
    }

    ElmerVtu.label = "Elmer Structural"
    ElmerVtu.file_path = join(fea_path, "Results", "case_t0001.vtu")

    output.struct.meshsolution = ElmerVtu.build_meshsolution()

    return True
